# blk-snapshot: block-based snapshots for linux

blk-snapshot is a kernel module and set of userspace utilities that allow you to
take both full and incremental snapshots of block devices.  Similar to how you
might use ``dd`` to create an image of a device, except the snapshot doesn't
change while you read it.

Each disk modification is recorded, giving you the ability to extract only the
blocks that changed since the last snapshot.

The kernel module lives in the Block I/O layer below the Filesystem. So you can
use this with any filesystem you happen to be running without modifying your
currently running kernel. All that is required is to load the module and tell it
which block devices you wish to track.

A snapshot is exposed as a block device. You can manipulate it as you would any
other block device using read(), write(), ``dd``, etc.

## Dependence

Packages required to build blk-snapshot:

Debian 7/8, Ubuntu 12.04-16.10, any other debian/ubuntu-based OS:
	# apt-get install make gcc python-docutils libblkid-dev linux-headers-$(uname -r)
RedHat Enterprise Linux, CentOS, Oracle Linux and any other RHEL-based OS:
	# yum install make gcc python-docutils libblkid-devel kernel-devel zlib-devel
SUSE Linux Enterprise (Server), OpenSUSE and any other SUSE-based OS:
	# zypper install make gcc python-docutils libblkid-devel kernel-devel

## Getting Started

To compile all of the things, type: ::

    $ make

You can install both the module and userspace utilities with: ::

    $ make install      (this will require root privledges)

This will install the kernel module for your currently running kernel and two
userspace utilities `bsctl` and `bsrw` to `/usr/local/bin`. Manpages are
included, you can get addition information by typing: ::

    $ man bsctl
    $ man bsrw

To load the kernel module: ::

    $ depmod -a
    $ modprobe blk-snapshot

## Taking Snapshots

To attach to a block device, say `/dev/sda1`, do the following: ::

    $ bsctl -a /dev/sda1

You can initiate a snapshot with `bsctl` using the `-s, --snapshot` option like
so: ::

    $ bsctl -s /dev/sda1

To take a full (or base) snapshot of this device: ::

    $ bsrw /dev/sda1 /tmp/sda1-snapshot.img

It is highly advised that you create the snapshot file on a partition NOT under
protection by blk-snapshot.

To end the snapshot: ::

    $ bsctl -e /dev/sda1

To take an incremental snapshot: ::

    $ bsctl -s /dev/sda1
    $ bsrw -i /dev/sda1 /tmp/sda1-snapshot.img
    $ bsctl -e /dev/sda1

This will overlay only the changed blocks onto the existing disk image. You can
verify the contents of the image by mounting the device: ::

    $ mkdir /tmp/sda1-mount
    $ mount -o loop /tmp/sda1-snapshot.img /tmp/sda1-mount

The contents of `/tmp/sda1-mount` should be roughly identical to that of
`/dev/sda1`.

Unmount the snapshot using: ::

    $ umount /tmp/sda1-mount

## Compile Options

If you're compiling the module, the defaults should be all that is necessary for
a normal setup. If you need something additional, here are the current options:

### Debug Options

* **BLKSNAP_DEBUG**: Enable debuging in blksnap-main.c
* **BLKSNAP_LOCK_DEBUG**: Enable lock debugging. **WARNING**: massive output
* **BLKSNAP_COMMON_DEBUG**: Enable debugging in blksnap-common.c
* **BLKSNAP_DATA_DEBUG**: Enable debugging in blksnap-cows.c
* **BLKSNAP_STORE_DEBUG**: Enable debugging in blksnap-store.c
* **BLKSNAP_NO_CAP_CHECK**: Disable IOCTL permission checking

These can be enabled at compile time with: ::

    $ make KCFLAGS="-DBLKSNAP_OPTION -DBLKSNAP_ANOTHER -DBLKSNAP_..."

### String Overrides

* **BLKSNAP_LOG_PREFIX**: Change the prefix string for log messages. Default is
  "blksnap".
* **BLKSNAP_LICENSE**: Change the module license, never use this.

These can be enabled at compile time with: ::

    $ make KCFLAGS="-DBLKSNAP_LICENSE=\\\"Some License\\\" ..."

### Other Overrides

* **DESTDIR**: The installation directory, default is "/usr/local".
* **KERNELDIR**: The path to your kernel headers.
* **MODULENAME**: Override the name of the module, default is "blk-snapshot"

These can be enabled at compile time with: ::

    $ make DESTDIR=/usr MODULENAME="my-module-name" KERNELDIR=/some/path

* **DONT_CLOBBER_VERSION_FILE**: Avoid overwritting the version string in
  version.h during compilation.

  The version files can be created independently using the `version` make
  target, like so: ::

    $ make version
    $ DONT_CLOBBER_VERSION_FILE=1 make

## Command Line Parameters

* **disabled**: Set to '1' to completely disable the module.
* **devices**: Set to a comma separated list of block devices to attach to.

These options are passed to the module either at module load time or to the
kernel before boot. If you pass these to the module at load time: ::

    $ modprobe blk-snapshot disabled=0 devices=/dev/sda1,/dev/sda2,/dev/sdb1

If you choose to pass these options to the kernel before boot, augment the
*linux* line with something like: ::

    linux ... root=/dev/sda1 ro blk-snapshot.disabled=1 blk-snapshot.devices=/dev/sda1
