#!/bin/sh
#####################################################################
# This script attempts to gather troubleshooting information on 
# a variety of Linux hosts.  Files that exist in one distribution of
# Linux may not exist in another distribution (SuSE vs Red Hat).
# Please ignore any errors reported about files not existing.
#
#####################################################################
#ScriptVER="1.5"
#
#####################################################################

####################################################################
# Test to see if run with with root permissions
#####################################################################
if test `id -u` != "0"
then
   /bin/echo "This script must be run with root permissions.  Please rerun as root or with sudo."
   exit
fi

####################################################################
# logger entry to identify start of script
#####################################################################
which logger > /dev/null 2>&1
if test $? -eq 0
then
   logger "linux_info.sh starting with PID $$"
fi

#####################################################################
# Create temporary directory using HOSTNAME-DateTime to store output
#####################################################################
CurDate=`/bin/date +%y%m%d_%H%M%S`
HostName=`uname -n`
LOGNAME="$HostName-$CurDate"
LOGDIR="/tmp/$LOGNAME"
/bin/echo "Log data will be stored in $LOGDIR"
/bin/echo "Output results will be tar-zipped to $LOGDIR.tgz"
/bin/echo "The Output File could be over 5MB if the file "
/bin/echo "Exceeds the Maximum file capacity of your email client "
/bin/echo "Please Request an FTP site form AppAssure Tech Support "
/bin/echo
mkdir $LOGDIR
mkdir $LOGDIR/script
cd $LOGDIR
touch $LOGDIR/script/ScriptVersion.$ScriptVER
mkdir $LOGDIR/OS

#####################################################################
# Gather /sys data
#####################################################################
if test -d /sys
then
   /bin/echo -n "Gathering /sys files: "
   ls -alRF /sys > $LOGDIR/OS/ls_sys.out
   mkdir -p $LOGDIR/sys/class
   mkdir $LOGDIR/sys/class/scsi_host
   mkdir $LOGDIR/sys/class/fc_host
   mkdir $LOGDIR/sys/class/iscsi_host
   mkdir $LOGDIR/sys/class/net
   cp -R /sys/class/scsi_host/*/  $LOGDIR/sys/class/scsi_host  2>  $LOGDIR/sys/copy_err.log
   cp -R /sys/class/fc_host/*/    $LOGDIR/sys/class/fc_host    2>> $LOGDIR/sys/copy_err.log
   cp -R /sys/class/iscsi_host/*/ $LOGDIR/sys/class/iscsi_host 2>> $LOGDIR/sys/copy_err.log
   cp -R /sys/class/net/*/        $LOGDIR/sys/class/net        2>> $LOGDIR/sys/copy_err.log
   mkdir -p $LOGDIR/sys/module
   cp -R /sys/module/scsi*        $LOGDIR/sys/module 2>> $LOGDIR/sys/copy_err.log
   cp -R /sys/module/qis*         $LOGDIR/sys/module 2>> $LOGDIR/sys/copy_err.log
   cp -R /sys/module/qla*         $LOGDIR/sys/module 2>> $LOGDIR/sys/copy_err.log
   cp -R /sys/module/qlge         $LOGDIR/sys/module 2>> $LOGDIR/sys/copy_err.log
   cp -R /sys/module/ixgbe        $LOGDIR/sys/module 2>> $LOGDIR/sys/copy_err.log
   cp -R /sys/module/nx_nic       $LOGDIR/sys/module 2>> $LOGDIR/sys/copy_err.log
   cp -R /sys/module/netxen_nic   $LOGDIR/sys/module 2>> $LOGDIR/sys/copy_err.log
   cp -R /sys/module/8021q        $LOGDIR/sys/module 2>> $LOGDIR/sys/copy_err.log
   cp -R /sys/module/bonding      $LOGDIR/sys/module 2>> $LOGDIR/sys/copy_err.log
   /bin/echo "... done"
fi

#####################################################################
# Gather /proc data
#####################################################################
/bin/echo -n "Gathering /proc files: "
mkdir $LOGDIR/proc
PROC_FILES="/proc/modules \
/proc/pci \
/proc/interrupts \
/proc/cmdline \
/proc/buddyinfo \
/proc/cpuinfo \
/proc/partition \
/proc/iomem \
/proc/meminfo \
/proc/slabinfo"

for file in $PROC_FILES
do
   if test -f $file
   then
      cp $file $LOGDIR/proc
   fi
done

cat /proc/scsi/scsi >$LOGDIR/proc/SCSI_Devices.txt
#####################################################################
# Gather network info for FCoE and Netxen
#####################################################################
mkdir $LOGDIR/proc/net
# Make sure /proc/net/nx_nic directory exists before gathering data
if test -d /proc/net/nx_nic
then
   cp -R /proc/net/nx_nic $LOGDIR/proc/net
   if test -f /proc/net/dev_mcast
   then
      cp /proc/net/dev_mcast $LOGDIR/proc/net
   fi
fi
# Make sure /proc/net/bonding directory exists before gathering data
if test -d /proc/net/bonding
then
   cp -R /proc/net/bonding $LOGDIR/proc/net
fi
# Make sure /proc/net/vlan directory exists before gathering data
if test -d /proc/net/vlan
then
   cp -R /proc/net/vlan $LOGDIR/proc/net
fi

# Make sure /proc/scsi directory exists before gathering data
if test -d /proc/scsi
then
   cd $LOGDIR/proc
   cp -R /proc/scsi $LOGDIR/proc
   cd $LOGDIR
fi
/bin/echo "... done"

#####################################################################
# Gather /etc data
#####################################################################
/bin/echo -n "Gathering /etc files: "
mkdir $LOGDIR/etc
ETC_FILES="/etc/modules.* \
/etc/modprobe.* \
/etc/qla*.conf \
/etc/hba.conf \
/etc/sysconfig/kernel \
/etc/sysconfig/hwconf \
/etc/mtab \
/etc/fstab"

for file in $ETC_FILES
do
   if test -f $file
   then
      cp $file $LOGDIR/etc
   fi
done

ls -alRF /etc/rc.d/ > $LOGDIR/etc/ls_etcrcd.out
/bin/echo "... done"

#####################################################################
# Gather module info
#####################################################################
/bin/echo -n "Gathering module information: "
mkdir $LOGDIR/modules
KERN_VER=`uname -r`
MAJ_VER=`/bin/echo $KERN_VER | head -c3`
if test $MAJ_VER = "2.4"
then
   EXT=".o"
fi
if test $MAJ_VER = "2.6"
then
   EXT=".ko"
fi

find /lib/modules/$KERN_VER/ -name qla\*$EXT -print \
  -exec modinfo {} \; -exec /bin/echo \; > $LOGDIR/modules/modinfo.out 2>&1

find /lib/modules/$KERN_VER/ -name qisioctl\*$EXT -print \
  -exec modinfo {} \; -exec /bin/echo \; >> $LOGDIR/modules/modinfo.out 2>&1
find /lib/modules/$KERN_VER/ -name qioctlmod\*$EXT -print \
  -exec modinfo {} \; -exec /bin/echo \; >> $LOGDIR/modules/modinfo.out 2>&1
find /lib/modules/$KERN_VER/ -name netxen\*$EXT -print \
  -exec modinfo {} \; -exec /bin/echo \; >> $LOGDIR/modules/modinfo.out 2>&1
find /lib/modules/$KERN_VER/ -name nx_\*$EXT -print \
  -exec modinfo {} \; -exec /bin/echo \; >> $LOGDIR/modules/modinfo.out 2>&1
find /lib/modules/$KERN_VER/ -name qlge\*$EXT -print \
  -exec modinfo {} \; -exec /bin/echo \; >> $LOGDIR/modules/modinfo.out 2>&1

# Gather qisioctl and qioctlmod info (if installed)
modinfo qisioctl  >> $LOGDIR/modules/qisioctl.out  2>> $LOGDIR/script/misc_err.log
modinfo qioctlmod >> $LOGDIR/modules/qioctlmod.out 2>> $LOGDIR/script/misc_err.log


ls -alRF /lib/modules/ > $LOGDIR/modules/ls_libmodules.out

lsmod > $LOGDIR/modules/lsmod.out
/bin/echo "... done"

#####################################################################
# Gather ethernet info
#####################################################################
/bin/echo -n "Gathering ethernet info: "
mkdir $LOGDIR/network
iptables --list > $LOGDIR/network/iptables.out
ifconfig -a > $LOGDIR/network/ifconfig.out
netstat -rn > $LOGDIR/network/netstat.out
which ethtool > /dev/null 2>&1
if test $? -eq 0;
then
   ETHDEVS=`grep "Link encap" $LOGDIR/network/ifconfig.out | cut -d " " -f1`
#   ETHDEVS=`/bin/ls $LOGDIR/sys/class/net 2>> $LOGDIR/script/misc_err.log`
   for file in $ETHDEVS
   do
      DRIVER=`ethtool -i $file 2>> $LOGDIR/script/misc_err.log | egrep "nx_nic|netxen_nic|qla|ixgbe|qlge"`
      if test $? -eq 0
      then
         ethtool -i $file > $LOGDIR/network/ethtool-i.$file
         ethtool    $file > $LOGDIR/network/ethtool.$file
         ifconfig   $file > $LOGDIR/network/ifconfig.$file
         netstat -tupl $file > $LOGDIR/network/internetservices.$file
         netstat -tup $file > $LOGDIR/network/activeconnections.$file
         netstat -nr $file > $LOGDIR/network/routes.$file
# DG add netxen /opt/netxen/nxflash components here including version info (-v option)
      fi
   done
fi

/bin/echo "... done"

#####################################################################
# Gather misc info
#####################################################################
/bin/echo -n "Gathering miscellaneous system info: "
mkdir $LOGDIR/misc
uptime                > $LOGDIR/misc/uptime.out 2>&1
lspci -v              > $LOGDIR/misc/lspci.out 2>&1
if test -x /usr/sbin/dmidecode
then
   dmidecode          > $LOGDIR/misc/dmidecode.out 2>&1
fi
df -lk                > $LOGDIR/misc/df.out 2>&1
fdisk -l              > $LOGDIR/misc/fdisk.out 2>&1
ps -ewf               > $LOGDIR/misc/ps.out
ls -lF /usr/bin/*gcc* > $LOGDIR/misc/gcc.out
/bin/echo "========" >> $LOGDIR/misc/gcc.out
chkconfig --list      > $LOGDIR/misc/chkconfig.out
ls -alRF /usr/lib/    > $LOGDIR/misc/ls_usrlib.out
if test -d /usr/lib64
then
   ls -alRF /usr/lib64/ > $LOGDIR/misc/ls_usrlib64.out
fi
which lsscsi > /dev/null 2>&1
if test $? -eq 0;
then
   lsscsi             > $LOGDIR/misc/lsscsi.out
   lsscsi --verbose   > $LOGDIR/misc/lsscsi_verbose.out
fi
sysctl -a             > $LOGDIR/misc/sysctl.out 2>&1
lsof                  > $LOGDIR/misc/lsof.out 2>&1
vmstat -a             > $LOGDIR/misc/vmstat.out 2>&1
free                  > $LOGDIR/misc/free.out 2>&1
/bin/echo "... done"

#####################################################################
# Gather System Log info
#####################################################################
/bin/echo -n "Gathering syslog info: "
blkid >/var/log/BLKID.log
mkdir $LOGDIR/logs
LOG_FILES="/var/log/messages* \
/var/log/localmessages* \
/var/log/warn* \
/var/log/dmesg* \
/var/log/boot* \
/var/log/aavdisk/* \
/var/log/appassure* \
/var/log/apprecovery* \
/var/log/BLKID.log \
/root/.config/appassure/* \
/root/.config/apprecovery/*"

for file in $LOG_FILES
do
   if test -f $file
   then
      cp $file $LOGDIR/logs
   fi
done

dmesg > $LOGDIR/logs/dmesg.out
/bin/echo "... done"

#####################################################################
# Gather OS info
#####################################################################
/bin/echo -n "Gathering OS info: "
OS_FILES=`ls /etc/*-release /etc/*_version 2>> $LOGDIR/script/misc_err.log`
for file in $OS_FILES
do
   if test -f $file
   then
      cp $file $LOGDIR/OS
   fi
done

rpm -qa  > $LOGDIR/OS/rpm_list.out
uname -a > $LOGDIR/OS/uname
/bin/echo "... done"

#####################################################################
# Gather boot info
#####################################################################
/bin/echo -n "Gathering boot info: "
mkdir $LOGDIR/boot
BOOT_FILES="/boot/grub/grub.conf \
/boot/grub/menu.lst \
/boot/efi/efi/SuSE/elilo.conf \
/etc/lilo.conf \
/etc/elilo.conf"

for file in $BOOT_FILES
do
   if test -f $file
   then
      cp $file $LOGDIR/boot
   fi
done

if test -f /etc/grub.conf
then
   cp /etc/grub.conf $LOGDIR/boot/etc_grub.conf
fi

ls -alRF /boot > $LOGDIR/boot/ls_boot.out
/bin/echo "... done"

#####################################################################
# Place holder below for dashboard.sh
#####################################################################
# Start of dashboard.sh
#####################################################################
# Post-process script output into pdeudo-meaningful html
#####################################################################

# The next line (LOGDIR=$1) must be removed before merging in with 
# the main data gathering script
cd $LOGDIR

#####################################################################
# Create revisionhistory.txt for the script output
#####################################################################
cat > $LOGDIR/script/revisionhistory.txt <<!
Linux Information Gathering Script - Revision History

  Rev  1.00.01  2009/11/16
      - Start of Revision History
      - Copied Script qla_linux_info.sh and modified to fit my needs 
      -- used the above noted script due to the parsing feature
      --- output now is in a neat HTML package 
      !

#####################################################################
# Create $LOGDIR/dashboard.html for the script output item stonlen and modified 
# form the above noted script
#####################################################################
#
# Header
#
cat > $LOGDIR/dashboard.html <<!
<head><title> Linux Information Gathering Script - Dashboard </title></head> 
<body> 
<font face="Courier New"> 
 <a id="top"></a> 
<div align="center"> 
<b> Linux Information Gathering Script Dashboard </b><br> 
!
/bin/echo `date` >> $LOGDIR/dashboard.html
/bin/echo "<hr><hr></div>" >> $LOGDIR/dashboard.html

#
# Header
#
cat >> $LOGDIR/dashboard.html <<!
<pre> 
<b>Index:</b><hr> 
Dashboard Links:                                      File Details:
!
/bin/echo -n "<a href=\"#systeminfo\">System Information</a>                                    " >> $LOGDIR/dashboard.html
/bin/echo    "<a href=\"details.html\">details.html</a>  - detailed information on collected files" >> $LOGDIR/dashboard.html
/bin/echo -n "<a href=\"#iscsiinfo\">iSCSI Information</a>                                     " >> $LOGDIR/dashboard.html
/bin/echo    "<a href=\"misc/lspci.out\">lspci.out</a>     - list of installed PCI/PCIe hardware" >> $LOGDIR/dashboard.html
/bin/echo -n "<a href=\"#etherinfo\">Ethernet Information</a>                                  " >> $LOGDIR/dashboard.html
/bin/echo    "<a href=\"modules/lsmod.out\">lsmod.out</a>     - list of loaded modules" >> $LOGDIR/dashboard.html
/bin/echo -n "<a href=\"#fclogs\">Fibre Channel Message Logs</a>                            " >> $LOGDIR/dashboard.html
if test -f misc/lsscsi.out
   then
      /bin/echo "<a href=\"misc/lsscsi.out\">lsscsi.out</a>    - list of scsi devices" >> $LOGDIR/dashboard.html
   elif test -f proc/scsi/scsi
   then
      /bin/echo "<a href=\"proc/scsi/scsi\">scsi</a>          - list of scsi devices (/proc/scsi/scsi)" >> $LOGDIR/dashboard.html
   else
      /bin/echo "scsi          - No SCSI information found" >> $LOGDIR/dashboard.html
fi
/bin/echo -n "<a href=\"#iscsilogs\">iSCSI Message Logs</a>                                    " >> $LOGDIR/dashboard.html
if test -f etc/modprobe.conf.local
   then
      /bin/echo "<a href=\"etc/modprobe.conf.local\">modprobe.conf</a> - list of module parameters (modprobe.conf.local)" >> $LOGDIR/dashboard.html
   elif test -f etc/modprobe.conf
   then
      /bin/echo "<a href=\"etc/modprobe.conf\">modprobe.conf</a> - list of module parameters" >> $LOGDIR/dashboard.html
   else
      /bin/echo "<a href=\"etc/modules.conf\">modules.conf</a>  - list of module parameters" >> $LOGDIR/dashboard.html
fi
/bin/echo "<a href=\"#etherlogs\">Ethernet Message Logs</a><br>" >> $LOGDIR/dashboard.html 

#
# System Information
#
cat >> $LOGDIR/dashboard.html <<!
<hr><a id="systeminfo"></a><b><a href="details.html#osinfo">System Information:</a></b>     <a href="#top">top</a><hr> 
!
HOSTNAME=`cut -d " " -f2 < $LOGDIR/OS/uname`
/bin/echo "Host Name:                 $HOSTNAME" >> $LOGDIR/dashboard.html
if test -f $LOGDIR/OS/redhat-release
then
   OSNAME=`cat $LOGDIR/OS/redhat-release`
elif test -f $LOGDIR/OS/SuSE-release
then
   OSNAME=`grep -i suse $LOGDIR/OS/SuSE-release`
   OSVER=`grep VERSION $LOGDIR/OS/SuSE-release`
   OSPATCH=`grep -h PATCH $LOGDIR/OS/*release`
else
   OSNAME="unknown"
fi
/bin/echo "OS Name:                   $OSNAME" >> $LOGDIR/dashboard.html
if test -f $LOGDIR/OS/SuSE-release
then
   /bin/echo "OS Version:                $OSVER,   $OSPATCH" >> $LOGDIR/dashboard.html
fi
KERNELVERSION=`cut -d " " -f3 < $LOGDIR/OS/uname`
/bin/echo "Kernel Version:            $KERNELVERSION" >> $LOGDIR/dashboard.html
/bin/echo "GCC Version:               `grep GCC $LOGDIR/misc/gcc.out`" >> $LOGDIR/dashboard.html
/bin/echo "System Up Time:           `cat $LOGDIR/misc/uptime.out`" >> $LOGDIR/dashboard.html
/bin/echo "System Manufacturer:       Unknown" >> $LOGDIR/dashboard.html
/bin/echo "System Model:              Unknown" >> $LOGDIR/dashboard.html
CPUMODEL=`grep "model name" $LOGDIR/proc/cpuinfo |uniq |cut -d " " -f3-9`
CPUCOUNT=`grep -c "model name" $LOGDIR/proc/cpuinfo`
CPUSPEED=`grep "cpu MHz" $LOGDIR/proc/cpuinfo |uniq |cut -d " " -f3-9`
/bin/echo "CPU Info:                  (x$CPUCOUNT) $CPUMODEL, $CPUSPEED MHz" >> $LOGDIR/dashboard.html
/bin/echo "BIOS Version:              Unknown" >> $LOGDIR/dashboard.html
/bin/echo >> $LOGDIR/dashboard.html
/bin/echo "<B>FC Driver Parameters:</B>" >> $LOGDIR/dashboard.html
grep -h options $LOGDIR/etc/mod* | grep qla2 >> $LOGDIR/dashboard.html
/bin/echo "<B>iSCSI Driver Parameters:</B>" >> $LOGDIR/dashboard.html
grep -h options $LOGDIR/etc/mod* | grep qla4 >> $LOGDIR/dashboard.html
/bin/echo "<B>Ethernet Module Parameters:</B>" >> $LOGDIR/dashboard.html
grep -h options $LOGDIR/etc/mod* | grep qla3 >> $LOGDIR/dashboard.html
grep -h options $LOGDIR/etc/mod* | grep qlge >> $LOGDIR/dashboard.html
grep -h options $LOGDIR/etc/mod* | grep nx_nic >> $LOGDIR/dashboard.html
grep -h options $LOGDIR/etc/mod* | grep netxen_nic >> $LOGDIR/dashboard.html
/bin/echo "<B>SCSI Module Parameters:</B>" >> $LOGDIR/dashboard.html
grep -h options $LOGDIR/etc/mod* | grep scsi >> $LOGDIR/dashboard.html
/bin/echo >> $LOGDIR/dashboard.html
/bin/echo "<B>QLogic HBAs Installed:</B>" >> $LOGDIR/dashboard.html
TESTHBA=`grep QLogic $LOGDIR/misc/lspci.out|grep Fibre`
if test -n "$TESTHBA"
then
   grep QLogic $LOGDIR/misc/lspci.out|grep Fibre >> $LOGDIR/dashboard.html
   FCINSTALLED=1
else
   /bin/echo "No QLogic Fibre Channel Adapters Installed" >> $LOGDIR/dashboard.html
   FCINSTALLED=0
fi
TESTHBA=`grep QLogic $LOGDIR/misc/lspci.out|grep "Network controller"`
if test -n "$TESTHBA"
then
   grep QLogic $LOGDIR/misc/lspci.out|grep "Network controller" >> $LOGDIR/dashboard.html
   ISCSIINSTALLED=1
else
   /bin/echo "No QLogic iSCSI Adapters Installed" >> $LOGDIR/dashboard.html
   ISCSIINSTALLED=0
fi
ETHERINSTALLED=0
TESTHBA=`grep network $LOGDIR/misc/lspci.out|grep "Ethernet controller"`
if test -n "$TESTHBA"
then
   grep network $LOGDIR/misc/lspci.out|grep "Ethernet controller" >> $LOGDIR/dashboard.html
   ETHERINSTALLED=1
fi
TESTHBA=`grep eth $LOGDIR/misc/lspci.out|grep "Ethernet controller"`
if test -n "$TESTHBA"
then
   grep eth $LOGDIR/misc/lspci.out|grep "Ethernet controller" >> $LOGDIR/dashboard.html
   ETHERINSTALLED=1
fi

TESTHBA=`grep LSI $LOGDIR/misc/lspci.out `
if test -n "$TESTHBA"
then
   grep LSI $LOGDIR/misc/lspci.out|grep Fibre >> $LOGDIR/dashboard.html
   FCINSTALLED=1
else
   /bin/echo "No LSI Fibre Channel Adapters Installed" >> $LOGDIR/dashboard.html
   FCINSTALLED=0
fi
TESTHBA=`grep LSI $LOGDIR/misc/lspci.out|grep "Network controller"`
if test -n "$TESTHBA"
then
   grep LSI $LOGDIR/misc/lspci.out|grep "Network controller" >> $LOGDIR/dashboard.html
   ISCSIINSTALLED=1
else
   /bin/echo "No LSI iSCSI Adapters Installed" >> $LOGDIR/dashboard.html
   ISCSIINSTALLED=0
fi
ETHERINSTALLED=0
TESTHBA=`grep network $LOGDIR/misc/lspci.out|grep "Ethernet controller"`
if test -n "$TESTHBA"
then
   grep network $LOGDIR/misc/lspci.out|grep "Ethernet controller" >> $LOGDIR/dashboard.html
   ETHERINSTALLED=1
fi
TESTHBA=`grep eth $LOGDIR/misc/lspci.out|grep "Ethernet controller"`
if test -n "$TESTHBA"
then
   grep eth $LOGDIR/misc/lspci.out|grep "Ethernet controller" >> $LOGDIR/dashboard.html
   ETHERINSTALLED=1
fi


TESTHBA=`grep "Intel Corporation 82598" $LOGDIR/misc/lspci.out|grep "Ethernet controller"`
if test -n "$TESTHBA" -a $FCINSTALLED -eq 1
then
   grep "Intel Corporation 82598" $LOGDIR/misc/lspci.out|grep "Ethernet controller" >> $LOGDIR/dashboard.html
   ETHERINSTALLED=1
fi
if test $ETHERINSTALLED -eq 0
then
   /bin/echo "No QLogic Ethernet Adapters Installed" >> $LOGDIR/dashboard.html
fi
/bin/echo >> $LOGDIR/dashboard.html

#
# Fibre Channel Message Logs
#
cat >> $LOGDIR/dashboard.html <<!
<hr><a id="fclogs"></a><b><a href="details.html#loginfo">Fibre Channel Message Logs:</a></b>     <a href="#top">top</a><hr> 
!
grep qla2 $LOGDIR/logs/messages >> $LOGDIR/dashboard.html
/bin/echo >> $LOGDIR/dashboard.html

#
# Wrap it up
#
cat >> $LOGDIR/dashboard.html <<!
<hr><a id="bottom"></a> <a href="#top">top</a><hr> 
<br><br><br><br><br><br><br><br><br><br><br><br><br><br><br> 
</pre> 
</font> 
</body>
!

#####################################################################
# Place holder below for details.sh
#####################################################################
# Start of details.sh
#####################################################################
# Post-process script output into pseudo-meaningful html
#####################################################################

# The next line (LOGDIR=$1) must be removed before merging in with 
# the main data gathering script
cd $LOGDIR

#####################################################################
# Create details.html for the script output
#####################################################################
DTL=$LOGDIR/details.html
#
# Header
#
cat > $DTL <<!
<head><title>Linux Information Gathering Script - Details</title></head> 
<body> 
<font face="Courier New"> 
 <a id="top"></a> 
<div align="center"> 
<b> Linux Information Gathering Script Details </b><br> 
!
/bin/echo `date` >> $DTL
/bin/echo "<hr><hr></div>" >> $DTL

#
# Index
#
cat >> $DTL <<!
<pre>
<b>Index:</b><hr>
<a href="#about">About</a>
<a href="#osinfo">OS Information</a>
<a href="#etcfiles">/etc Information</a>
<a href="#modules">Module Information</a>
<a href="#procinfo">/proc Information</a>
<a href="#etherinfo">Ethernet Information</a>
<a href="#loginfo">System Log Information</a>
<a href="#miscinfo">Miscellaneous Information</a><br>
!

#
# About
#
cat >> $DTL <<!
<hr><a id="about"></a><b><a href="details.html">About:</a></b>     <a href="#top">top</a><hr>
This details file will walk through the information gathered by the information gathering script.

<a href="$LOGDIR/dashboard.html">$LOGDIR/dashboard.html</a>
This file is the starting place for all your basic troubleshooting needs.  It displays an overview
of the server.

<a href="details.html">details.html</a>
This file.

<a href="script/revisionhistory.txt">revisionhistory.txt</a>
This file contains the revision history for the Linux Information Gathering script. Please note that the parsing functionality is still under constructions and is in beta. Please use the folder contiend in the Archive to review the system information.

!

#
# OS Information
#
cat >> $DTL <<!
<hr><a id="osinfo"></a><b><a href="details.html">OS Information:</a></b>     <a href="#top">top</a><hr>
!
OSFILES=`ls OS`
for FILE in $OSFILES
do
   /bin/echo -n "<a href=\"OS/$FILE\">OS/$FILE</a>    " >> $DTL
done
/bin/echo >> $DTL
/bin/echo "These files include the OS release version, the running kernel version (uname), the installed rpms," >> $DTL
/bin/echo "and a list of files in the /sys partition (2.6 kernel only)." >> $DTL
/bin/echo >> $DTL

#
# Boot files
#
BOOTFILES=`ls boot`
for FILE in $BOOTFILES
do
   /bin/echo -n "<a href=\"boot/$FILE\">boot/$FILE</a>    " >> $DTL
done
/bin/echo >> $DTL
/bin/echo "These files include all boot configuration files and a list of files in the /boot directory." >> $DTL
/bin/echo >> $DTL

#
# /etc Information
#
cat >> $DTL <<!
<hr><a id="etcfiles"></a><b><a href="details.html">/etc Information:</a></b>     <a href="#top">top</a><hr>
!
ETCFILES="etc/modprobe.conf etc/modprobe.conf.local etc/modprobe.conf.dist etc/modules.conf etc/modules.conf.local etc/kernel"
for FILE in $ETCFILES
do
   if test -f $FILE
   then
      /bin/echo -n "<a href=\"$FILE\">$FILE</a>    " >> $DTL
   fi
done
/bin/echo >> $DTL
/bin/echo "These files are used to determine the order modules are loaded, specify optional module parameters," >> $DTL
/bin/echo "and determine which modules are included in the ramdisk image during bootup." >> $DTL
/bin/echo >> $DTL

ETCFILES="etc/qla2xxx.conf etc/qla2300.conf etc/qla2200.conf etc/hba.conf"
ATLEASTONEFILE=0
for FILE in $ETCFILES
do
   if test -f $FILE
   then
      ATLEASTONEFILE=1
      /bin/echo -n "<a href=\"$FILE\">$FILE</a>    " >> $DTL
   fi
done
if test $ATLEASTONEFILE -eq 1
then
   /bin/echo >> $DTL
   /bin/echo "The file qla*.conf, if present, is used to store an ascii representation of persistent binding and" >> $DTL
   /bin/echo "LUN masking as defined by SANsurfer or scli.  The file hba.conf, if present, points to the proper " >> $DTL
   /bin/echo "dynamic loadable library for the SNIA API (HBAAPI)." >> $DTL
   /bin/echo >> $DTL
fi

ETCFILES="etc/fstab etc/mtab"
for FILE in $ETCFILES
do
   if test -f $FILE
   then
      /bin/echo -n "<a href=\"$FILE\">$FILE</a>    " >> $DTL
   fi
done
/bin/echo >> $DTL
/bin/echo "These files determine which filesystem directories are attempted to mount and which filesystem " >> $DTL
/bin/echo "directories are actually mounted." >> $DTL
/bin/echo >> $DTL

/bin/echo -n "<a href=\"etc/ls_etcrcd.out\">etc/ls_etcrcd.out</a>    " >> $DTL
/bin/echo >> $DTL
/bin/echo "This file is a directory listing of all startup files performed at various runlevels at boot/shutdown" >> $DTL
/bin/echo >> $DTL

#
# Module Information
#
cat >> $DTL <<!
<hr><a id="modules"></a><b><a href="details.html">Module Information:</a></b>     <a href="#top">top</a><hr>
!
MODFILES="modules/ls_libmodules.out modules/lsmod.out modules/modinfo.out modules/qisioctl.out modules/qioctlmod.out"
for FILE in $MODFILES
do
   if test -f $FILE
   then
      /bin/echo -n "<a href=\"$FILE\">$FILE</a>    " >> $DTL
   fi
done
/bin/echo >> $DTL
/bin/echo "The file ls_libmodules.out is a list of all modules for the current running kernel.  The file " >> $DTL
/bin/echo "lsmod.out is a list of all currently loaded modules.  The file modinfo.out is a list of modinfo" >> $DTL
/bin/echo "output of all QLogic modules in the current running kernel." >> $DTL
/bin/echo >> $DTL

#
# /proc Information
#
cat >> $DTL <<!
<hr><a id="procinfo"></a><b><a href="details.html">/proc Information:</a></b>     <a href="#top">top</a><hr>
!
PROCFILES=`ls proc`
for FILE in $PROCFILES
do
   if test -f proc/$FILE
   then
      /bin/echo -n "<a href=\"proc/$FILE\">proc/$FILE</a>    " >> $DTL
   fi
done
/bin/echo >> $DTL
/bin/echo "These files include CPU information, running modules, and (optionally) pci information as     " >> $DTL
/bin/echo "reported in the /proc filesystem.                                                             " >> $DTL
/bin/echo >> $DTL

if test -d proc/scsi
then
   if test -f proc/scsi/scsi
   then
      /bin/echo "<a href=\"proc/scsi/scsi\">proc/scsi/scsi</a>    " >> $DTL
      /bin/echo "This is a list of all devices scanned by the SCSI module as reported in the /proc filesystem. " >> $DTL
      /bin/echo >> $DTL
   fi
fi

#
# Ethernet Information
#
cat >> $DTL <<!
<hr><a id="etherinfo"></a><b><a href="details.html">Ethernet Information:</a></b>     <a href="#top">top</a><hr>
!
/bin/echo -n "<a href=\"network/ifconfig.out\">network/ifconfig.out</a>  " >> $DTL
/bin/echo -n "<a href=\"network/netstat.out\">network/netstat.out</a>  " >> $DTL
/bin/echo "<a href=\"network/iptables.out\">network/iptables.out</a>" >> $DTL
/bin/echo "These files include network interface configurations and interface routing information." >> $DTL
/bin/echo >> $DTL
if test $ETHERINSTALLED -eq 1
then
   QLETHERDRIVER=0
   ETHDEVS=`grep "Link encap" $LOGDIR/network/ifconfig.out | cut -d " " -f1`
   for file in $ETHDEVS
   do
      if test -f $LOGDIR/network/ethtool-i.$file
      then
         QLETHERDRIVER=1    
         /bin/echo -n "<a href=\"network/ifconfig.$file\">network/ifconfig.$file</a>  " >> $DTL
         /bin/echo -n "<a href=\"network/ethtool-i.$file\">network/ethtool-i.$file</a>  " >> $DTL
         /bin/echo "<a href=\"network/ethtool.$file\">network/ethtool.$file</a>" >> $DTL
      fi
   done
   if test $QLETHERDRIVER -eq 1
   then
      /bin/echo "These files include details about specific network interfaces." >> $DTL
# DG bonding, netxen and vlan stuff goes here.
# Still need to add in the Netxen /proc/net/devX files in the format
# devX/file1 devX/file2 devX/file3 ... devX/file7
# devY/file1 devY/file2 devY/file3 ... devY/file7
# See ts80lx52/dev/sda13 and ts80lx56/dev/sdb1 for script outputs
   fi
fi 
/bin/echo >> $DTL

#
# System Log Information
#
cat >> $DTL <<!
<hr><a id="loginfo"></a><b><a href="details.html">System Log Information:</a></b>     <a href="#top">top</a><hr>
!
BOOTFILES=`ls logs/boot*`
for FILE in $BOOTFILES
do
   /bin/echo -n "<a href=\"$FILE\">$FILE</a>    " >> $DTL
done
/bin/echo >> $DTL
/bin/echo "Boot logs" >> $DTL
/bin/echo >> $DTL
MSGFILES=`ls logs/message*`
for FILE in $MSGFILES
do
   /bin/echo -n "<a href=\"$FILE\">$FILE</a>    " >> $DTL
done
/bin/echo >> $DTL
/bin/echo "System messages files" >> $DTL
/bin/echo >> $DTL
MSGFILES=`ls logs/* | grep -v "logs/message" | grep -v boot`
for FILE in $MSGFILES
do
   /bin/echo -n "<a href=\"$FILE\">$FILE</a>    " >> $DTL
done
/bin/echo >> $DTL
/bin/echo "Other log files" >> $DTL
/bin/echo >> $DTL

#
# Misc Information
#
cat >> $DTL <<!
<hr><a id="miscinfo"></a><b><a href="details.html">Miscellaneous Information:</a></b>     <a href="#top">top</a><hr>
!
/bin/echo -n "<a href=\"misc/fdisk.out\">misc/fdisk.out</a>     " >> $DTL
/bin/echo "<a href=\"misc/df.out\">misc/df.out</a>" >> $DTL
/bin/echo "List of devices recognized by the OS SCSI disk module and list of mounted disks." >> $DTL
/bin/echo >> $DTL
/bin/echo -n "<a href=\"misc/chkconfig.out\">misc/chkconfig.out</a>   " >> $DTL
/bin/echo "System runlevel configuration." >> $DTL
/bin/echo -n "<a href=\"misc/gcc.out\">misc/gcc.out</a>         " >> $DTL
/bin/echo "List of installed gcc binaries and version information." >> $DTL
/bin/echo -n "<a href=\"misc/lspci.out\">misc/lspci.out</a>       " >> $DTL
/bin/echo "List of hardware installed as recognized by <i>lspci -v</i>" >> $DTL
/bin/echo -n "<a href=\"misc/dmidecode.out\">misc/dmidecode.out</a>   " >> $DTL
/bin/echo "Lists Motherboard and BIOS information as recognized by <i>dmidecode</i>" >> $DTL
/bin/echo -n "<a href=\"misc/ps.out\">misc/ps.out</a>          " >> $DTL
/bin/echo "List of all running processes." >> $DTL
/bin/echo -n "<a href=\"misc/uptime.out\">misc/uptime.out</a>      " >> $DTL
/bin/echo "System uptime." >> $DTL
/bin/echo -n "<a href=\"misc/ls_usrlib.out\">misc/ls_usrlib.out</a>   " >> $DTL
/bin/echo "32-bit Loadable libraries" >> $DTL
if test -f misc/ls_usrlib64.out
then
   /bin/echo -n "<a href=\"misc/ls_usrlib64.out\">misc/ls_usrlib64.out</a> " >> $DTL
   /bin/echo "64-bit Loadable libraries" >> $DTL
fi
/bin/echo >> $DTL
OTHERMISCFILES="misc/lsscsi.out misc/lsscsi_verbose.out misc/sysctl.out misc/vmstat.out misc/free.out misc/lsof.out"
for FILE in $OTHERMISCFILES
do
   if test -f $FILE
   then
      /bin/echo -n "<a href=\"$FILE\">$FILE</a>    " >> $DTL
   fi
done
/bin/echo >> $DTL
/bin/echo "Other miscellaneous files listing various system resources." >> $DTL
/bin/echo >> $DTL

#
# Wrap it up
#
cat >> $DTL <<!
<hr><a id="bottom"></a> <a href="#top">top</a><hr> 
<br><br><br><br><br><br><br><br><br><br><br><br><br><br><br> 
</pre> 
</font> 
</body>
!


#####################################################################
# Special AppAssure Section
#####################################################################
find / |grep "/.blksnap" | egrep -v "data|changelog|metadata" | tar -cvzf $LOGDIR/blksnap.tgz --files-from -

#####################################################################
# Create compressed archive of results ... then clean up
#####################################################################
/bin/echo -n "Creating compressed archive and cleaning up ... "

# Temporary cleanup of $LOGDIR/sys to avoid extraction errors
cd $LOGDIR
if test -d ./sys
then
   tar czf $LOGDIR/OS/sys_files.tgz ./sys
   rm -rf $LOGDIR/sys
fi
# Now back to our regularly scheduled program

cd /tmp
tar cvzf $LOGNAME.tgz ./$LOGNAME*
if test $? -ne 0 
then
   /bin/echo "*!*! Error while archiving the support data."
   /bin/echo "     Please tar and compress $LOGDIR by hand"
   /bin/echo "     and Email it to support@qlogic.com"
else
   rm -rf /tmp/$LOGNAME
   /bin/echo "done"
   /bin/echo
   /bin/echo "Please Email the file: $LOGDIR.tgz to support@appassure.com"
   /bin/echo "Please note you ticket number in the email "
fi

#####################################################################
# All done ...
#####################################################################
exit
