#include <stdlib.h>
#include <errno.h>
#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include "bsctl-log.h"
#include "bsctl.h"

void print_usage(char *prog_name)
{
	printf("Usage: %s DEV_NAME\n", prog_name);
}

void print_err(char *description, struct bsctl_error *err)
{
	if (!err) {
		printf("%s: err pointer is NULL\n", description);
		return;
	}

	if (!err->err) {
		printf("%s: success\n", description);
		return;
	}

	printf("%s: err code: [%d] msg: [%s]\n", description, err->err, err->msg);
	return;
}

void print_device_info(struct bsctl_device_info *info)
{
	if (!info) {
		printf("%s: info is NULL\n", __func__);
		return;
	}

	printf(" name: %s\n device_name: %s\n snapshot_name = %s\n link_name = %s\n flags = %u\n",
	info->name, info->device_name, info->snapshot_name, info->link_name, info->flags);

	return;
}

int main(int argc, char *argv[])
{
	char *dev_path;
	struct bsctl_error *err;
	struct bsctl_device_info *info;

	if (argc < 2) {
		print_usage(argv[0]);
		return EINVAL;
	}

	dev_path = argv[1];

	err = malloc(sizeof(*err));
//	err = 0;
	bsctl_err_reset(err);

	printf("bsctl api version = %d\n", bsctl_api_version(err));

	bsctl_chlog_map(dev_path, err);
	print_err("bsctl_chlog_map", err);

	bsctl_err_reset(err);
	info = bsctl_device_info_get(dev_path, err);
	print_err("bsctl_device_info_get", err);
	print_device_info(info);



	bsctl_snapshot_cancel(dev_path, err);
	print_err("bsctl_snapshot_cancel", err);

	bsctl_err_reset(err);
	bsctl_detach(dev_path, err);
	print_err("bsctl_detach", err);

	free(err);
	return 0;
}
