#ifndef _XFS_PRINT_H_
#define _XFS_PRINT_H_

#if defined (__x86_64__)
	#define OFFTF "lx"
	#define SZOF  "lu"
	#define U64TF "lu"
	#define X64TF "lx"
#elif defined (__i386__)
	#define OFFTF "x"
	#define SZOF  "u"
	#define U64TF "llu"
	#define X64TF "llx"
#else
#error "unsupported arch type"
#endif

#define VALUE_POS "\r\t\t\t\t "
#define COMMENT_POS "\r\t\t\t\t\t\t "

#define PRE_XFS_PRINT(struct_pointer, struct_ondisk_offset)		\
	const typeof(*struct_pointer) *struct_ptr_ = struct_pointer;	\
	const uint64_t struct_ondisk_offset_ = struct_ondisk_offset

/* needs PRE_XFS_PRINT before usage */
#define XFS_PRINT(field, postfix, comment)				\
	xfs_print_##postfix(#field, &struct_ptr_->field, comment,	\
			    struct_ondisk_offset_ +			\
			    offsetof(typeof(*struct_ptr_), field))

void xfs_print_u8(char *field_name, const uint8_t *value, char *comment,
		  uint64_t offset);
void xfs_print_x8(char *field_name, const uint8_t *value, char *comment,
		  uint64_t offset);
void xfs_print_u16(char *field_name, const uint16_t *value, char *comment,
		   uint64_t offset);
void xfs_print_x16(char *field_name, const uint16_t *value, char *comment,
		   uint64_t offset);
void xfs_print_u32(char *field_name, const uint32_t *value, char *comment,
		   uint64_t offset);
void xfs_print_d32(char *field_name, const uint32_t *value, char *comment,
		   uint64_t offset);
void xfs_print_x32(char *field_name, const uint32_t *value, char *comment,
		   uint64_t offset);
void xfs_print_le_x32(char *field_name, const uint32_t *value, char *comment,
		   uint64_t offset);
void xfs_print_u64(char *field_name, const uint64_t *value, char *comment,
		   uint64_t offset);
void xfs_print_x64(char *field_name, const uint64_t *value, char *comment,
		   uint64_t offset);
void xfs_print_lsn(char *field_name, const uint64_t *value, char *comment,
		   uint64_t offset);
void xfs_print_magic(char *field_name, const uint32_t *magic, char *comment,
		     uint64_t offset);
void xfs_print_uuid(char *field_name, const uuid_t *uuid, char *comment,
		     uint64_t offset);
/* Non-compatible with XFS_PRINT */
void xfs_print_str(char *field_name, const char *str, char *comment,
		   uint64_t offset);

#endif /* _XFS_PRINT_H_ */
