This is the current state of blk-snapshot logic to process bio requests.

The road-map of global changes is:

+ simplify a logic of current blk-snapshot, reduce slightly a memory consumption, make some processes synchronous,

+ divide blocks tree to blocks tree and cow_data tree, use lockless technique for cow_data tree

+ change blocks tree to bitmap (store flags only in flat virtual bitmap array with holes)

+ process multi-segment user's write bio requests
  + throw user's write bio request to separate kthread and process them in user context
	+ synchronous process segment-by-segment large user's write bio requests

- finally implement an instant rollback

-+ process multi-segment snapshot bio requests
  + reverse bio_split() to allow process large bio requests w/o overusing stack with recursive bio_split() calls
	+ throw snapshot bio requests to separate kthread and process them in user context
	- process multi-segment bio requests

===============================================================================================================
1. A logic of current blk-snapshot: memory consumption is reduced,  bio processing is made synchronous
===============================================================================================================

Current implementation of blk-snapshot has two struct blksnap_bio_queue:
---------------------------------------------------------------

- write_queue
- read_snap_queue

Sources for write_queue:

.............................................................................................................
!!! TODO: actualise following context of this document according to current state of blk-snapshot project !!!
---------------------------------------------------------------
.............................................................................................................

-> copy_write(bio) - set bi_end_io to bio_end_copy() to process bio in CoW-chain
	-> bio_end_copy(bio) - check if bio was prcessed successfully and has good data
		-> blksnap_bio_kthread_enqueue(bio)
			-> bio_list_add(bio) - enqueues a current bio in a CoW-chain of bios

		-> bio_end_copy_errror(bp, bio) - if some error occurs when bio was processed
			-> blksnap_bio_kthread_enqueue(bp->next)
				-> bio_list_add(bp->next) - enqueues a next bio in CoW-chain

-> __block_update(bio)
	-> blksnap_bio_kthread_enqueue(bio)
		-> bio_list_add(bio) - enqueues a bio to write CoW block in data-store file

Sources of pending_reads:

-> snapshot_read_request(bio) - check if block is good to redrect a request to original device or to read data from CoW store
	-> bio_list_add(bio) - enqueues a current bio if CoW-block is IN_FLIGHT. It means that customer performs
				a write operation to this block of original device and the CoW-operation is not fnished yet

Why we need to use pending_reads bio_list in each claim? Loks like there is not often situation when requests
to write to original block N and to read from snapshot device block N are performed simultaneously.

The problem solved with pending_reads bio_list is that bio-requests in different CoW-chains and requests to read snapshot-device are preformed asynchronous.
It makes possible to read from snapshot device modified data if this request is performed during CoW-operation.

Using 2 different bio_list makes code more complexity and using pending_reads bio_list in each claim structure consumes
a lot of memory during snapshot runs.

=======================================================================================
Proposed 3 new techniques to solve this issue using single kthread and single bio_list:
=======================================================================================
1. Use single bio_list for all requests that must be processed in blk-snapshot, use asynchronous performing of bio-requests, use reinjecting of bio-requests
   if it can't be process on current step.

Type of processed bio-requests:
 - [WO] WRITE customer's bio request to original disk
 - [RC] READ CoW bio request to original disk (read original data for CoW)
 - [WC] WRITE CoW bio request to original disk (write original data to CoW-store)
 - [RS] READ bio request to snapshot device

All these bio-request types can be differences with bi_private and bi_end_io fields and bio_data_dir() status.
  [WO] - bio_data_dir() = WRITE, bi_private - original, bi_end_io - original
  [RC] - bio_data_dir() = READ, bi_private = blksnap_block, bi_end_io = cow_read_finish()
  [WC] - bio_data_dir() = WRITE, bi_private = blksnap_block, bi_end_io = cow_write_finish()
  [RS] - bio_data_dir() = READ, bi_private - original, bi_end_io - original

Customer's READ bio-requests to original disk is passed thru blk-snapshot w/o any process,
don't enqueue to our bio_list and can't be appear in this process workflow.

Proposed changes in existing structures:
blksnap_volume:
 - remove dirty, changed_while_detached, snapshot_errors
 - add unsigned int flags
 - add new values of flags
   VOLUME_DIRTY     - instead of dirty
   VOLUME_CHANGED   - instead of changed_while_detached
   SNAP_INCREMENTAL - true if customer requests and CAN get an
                      incremental snapshot (ONLY if DIRTY = true)
                      false if customer requests base snapshot
   SNAP_FAIL        - snapshot gets error and can't be continue and finished successfully

blksnap_block:
 - add new values of flags:
   COW_ON_DISK - if original data is already in data store
   COW_IN_MEM  - if original data is already in memory
 - rename value IN_FLIGHT to COW_IN_FLIGHT - if CoW operation is started but not finished

Proposed new structures:
cow_data:
 - atomic_t refcnt - reference counter, protect a blksnap_cow object to be removed when it is used
 - sector_t extent_offset - an offset in the extent map of data store
 - struct page *page - page of memory to temporary store a bio data [RC]/[WC]

Proposed workflow and logic:

1. Process [WO] request:
  For each 4k blocks:
    - if (SNAP_FAIL)
        process next block
    - if (NOT block in tree AND disk is NOT DIRTY AND snapshot is INCREMENTAL
        OR (disk is NOT DIRTY AND snapshot is INCREMENTAL AND NOT THIS_SNAP))
        process next block
    - if (COW_[ON_DISK|IN_MEM])
        process next block
    - if (COW_IN_FLIGHT)
        set POSTPONE
        process next block
    - else
        set COW_IN_FLIGHT
        allocate cow_data
        find a space in data store
        create RC request
        enqueue RC request
        set POSTPONE
        process next block

  If (POSTPONE)
        enqueue WO request
  Else
        run WO request

2. Process [RC] request in cow_read_finish():
  - if (error)
      set SNAP_FAIL
  - else
      get_page(bio_page(bio)) - it is needed to 
      cow_data->page = bio_page(bio)
      set COW_IN_MEM
      clear COW_IN_FLIGHT
      create WC request
      enqueue WC request

  clear COW_IN_FLIGHT
  finish bio

3. Process [WC] request in cow_write_finish():
  - if (error)
      restore WC request
      enqueue WC request
      ...
      to be continue...

