Blk-Snapshot Kernel Module Memory Requirements
==============================================

This document describes structures of blk-snapshot 7.x, 6.0.x and memory requirements for customer's servers.

blk-snapshot 7.x.x
------------------

### struct bitmap_chunk

Member         | Length, bytes | Type
---------------|---------------|-----------
changed_blocks | 2             | uint16_t
blocks         | 8             | uint8_t *
lock_alloc     | 40            | struct rw_semaphore

**Summary size for aligned struct : 56 bytes**

### blocks bitmap

Member   | Length, bytes | Type
---------|---------------|---------------
blocks[] | 4096          | uint8_t [4096]

*Each chunk represents 4089 disk blocks.*

**Summary size: 4096 bytes**

### struct blksnap_cow_block

Member        | Length, bytes | Type
--------------|---------------|--------------------------
addr          | 8             | sector_t
rb_node       | 24            | struct rb_node
data_lock     | 40            | struct mutex
claim         | 32            | struct blksnap_data_claim
pending_reads | 16            | struct bio_list

**Summary size: 120 bytes**

### The consumtion of memory for different mode of blk-snapshot

Disk State                               | Memory Consumption
-----------------------------------------|----------------------------------------------------------------------------
Disk is not attached                     | nothing
Disk is attached                         | 4159 bytes for each 4096 4k-block (16 MB) on the disk
Disk is attached and snapshot is getting | 120 bytes for each _changed_ 4k-block on the disk

### Memory used by bitmap

Disk size     | 1TB   | 2TB   | 5TB   | 10TB  | 20TB
--------------|-------|-------|-------|-------|-------
Attach        | 248MB | 495MB | 1.2GB | 2.5GB | 5GB

### Memory used by CoW tree

Changes  | 1 GB | 10 GB | 100 GB | 1TB  
---------|------|-------|--------|------
Snapshot | 29MB | 293MB | 2.9GB  | 29GB 

In snapshot mode blk-snapshot module uses CoW-storage that by default equals to 15%
of the whole size of disk, so we could calculate the maximum memory consumtion by blk-snapshot
for snapshotting.

### MAXIMAL MEMORY USAGE DURING SNAPSHOT

Disk size     | 1TB   | 2TB   | 5TB   | 10TB  | 20TB
--------------|-------|-------|-------|-------|------
CoW size      | 150GB | 300GB | 750GB | 1.5TB | 3TB
Blocks in CoW | 36.6M | 73M   | 183M  | 366M  | 732M
COW tree      | 4.4GB | 8.8GB | 22GB  | 44GB  | 88GB
Bitmap        | 0.2GB | 0.5GB | 1.2GB | 2.5GB | 5GB
Summary       | 4.6GB | 9.3GB | 23GB  | 47GB  | 93GB

**NOTE:** *new mode "incremental snapshot" in 7.x.x allows to perform CoW operation for blocks
that is in current snapshot only. This technique significantly reduce numbers of CoW
operations and reduce use of memory by blk-snapshot during snapshot.*

Compare memory usage 7.0.0 vs 6.1.2
-----------------------------------

Preconditions:
* Disk Size: 1TB
* CoW Data Store: 150GB
* Write File: 700GB

## Memory usage after write 700GB file on attached device
Version | Used Before | Cached | Used After | Cached  | Used Summary
--------|-------------|--------|------------|---------|-------------------
6.1.2   | 2.6 GB      | 2.1 GB | 15.4 GB    | 0.1 GB  | **15.0 GB**
7.0.0   | 0.3 GB      | 0.0 GB | 12.0 GB    | 9.8 GB  | **1.9 GB**

## Memory usage after write 145GB file during snapshot
Version | Used Before | Cached | Used After | Cached  | Used Summary
--------|-------------|--------|------------|---------|-------------------
6.1.2   | 2.4 GB      | 2.1 GB | 15.3 GB    |  6.8 GB | **8.2 GB**
7.0.0   | 0.4 GB      | 0.1 GB | 15.1 GB    | 14.3 GB | **1.1 GB**

**NOTE:** *this is "integral" compare, that doesn't separate system memory usage and blk-snapshot memory usage, so present result is just an indicatitive.*

blk-snapshot 6.x.x
------------------

### struct blksnap_block

Member        | Length, bytes | Type
--------------|---------------|--------------------------
addr          | 8             | sector_t
flags         | 8             | unsigned long
rb_node       | 24            | struct rb_node
cow_data      | 8             | struct blksnap_cow_data \*

**Summary size: 48 bytes**

### struct blksnap_cow_data

Member         | Length, bytes | Type
---------------|---------------|--------------------------
lock           | 40            | struct mutex
claim          | 40            | \* + struct blksnap_data_claim
pendings_reads | 16            | struct bio_list

**Summary blksnap_cow_data size: 96 bytes**

### The consumtion of memory for different mode of blk-snapshot

Disk State                               | Memory Consumption
-----------------------------------------|-------------------------------------------------------------
Disk is not attached                     | nothing
Disk is attached                         | 48 bytes for each _changed_ 4k-block on the disk
Disk is attached and snapshot is getting | 48 + 96 = 144 bytes for each _changed_ 4k-blocks on the disk

### Memory usage in Attach/Snapshot Mode

Changes  | 1 GB | 10 GB | 100 GB | 1TB
---------|------|-------|--------|-----
Attach   | 12MB | 117MB | 1.2GB  | 11.7GB
Snapshot | 35MB | 352MB | 3.5GB  | 35GB

In snapshot mode blk-snapshot modules uses CoW-storage that by default equals tp 15%
of the whole size of disk, so we could calculate the maximum memory consumtion by blk-snapshot
for snapshotting:

### MAXIMAL MEMORY USAGE DURING SNAPSHOT

Disk size     | 1TB   | 2TB   | 5TB   | 10TB  | 20TB
--------------|-------|-------|-------|-------|------
CoW size      | 150GB | 300GB | 750GB | 1.5TB | 3TB
Blocks in CoW | 39M   | 78M   | 196M  | 393M  | 786M
Changelog     | 10GB  | 20GB  | 50GB  | 100GB | 200GB
COW nodes     | 5.3GB | 11GB  | 26GB  | 53GB  | 105GB
Summary       | 15GB  | 30GB  | 76GB  | 152GB | 304GB

Use cases
---------

Say customer have 2 1TB disks and 16GB of memory.
The write flow is 1MB/second for both disks. If we assume blk-snapshot module shouldn't
use more than 50% of memory of this server, we could calculate a maximal period of snapshot getting
for this server.

    16MB * 50% = 8GB (memory available for blk-snapshot)
    1GB / (1MB * 2) = 8.5 minutes (time to change 1GB on both disks)
    8GB / 15MB = 546 (multiplier, the sense - writing of 546GB to both disks causes blk-snapshot to use 8GB of memory)
    8.5 * 546 = 77 hours. The maximal period between snapshots on this server.

The common formula
------------------

    Tmax =~ 19 * Mmax / V
        where:
            19   - precalculated multiplier
            Mmax - planned maximal memory usage by blk-snapshot (in GB)
            V    - sum of medium write flow speed for all protected disks (in MBps)
            Tmax - maximal time interval between snapshots (in hours)
