Bitmap locking
====================

1. Protection from referencing to an unallocated chunk
--------------------
### Potential threat
Kernel panic on segmentation fault

### Shared resource
struct bitmap_chunk {
	...
	block_bm_t *blocks;
	...
}

### Critical section
Every dereference of chunk->blocks pointer

### Security means
Sinchronization mechanism: rw-semaphore
Declaration:
	struct bitmap_chunk {
		...
		struct rw_semaphore lock_alloc;
	};

### Example of race condition
Write BIO-request has arrived. Chunk which contains corresponding block is
allocated already.
It's need to set bit NEXT_CHUNK for the block in bitmap chunk.
When the thread reaches code of setting bit, the other thread has already freed
the chunk. Dereference of &i->chunk->blocks[i.nblock] pointer causes
segmentation fault.

### Why we shouldn't use rw-spinlock
... For now I'm not shure about my proofs.

2. Protection from losing bit of block state
--------------------
### Potential threat
Corruption of changelog, ergo: corrupted snapshots

### Shared resource
bitmap.chunk[i]->blocks[j]

### Critical section
Setting of bit

### Security means
Atomic bitwise operations from bitops.h

### Example of race condition
Let a = bitmap.chunk[1].blocks[33]
Assume initial value of a is 00010010 in binary
Critical section of code:
	a |= 1 << bit
which consists of these ops:
	load a
	b = 1 << bit
	a = a | b
Sequence which causes race:
thread 1 (bit = 0)	| thread 2 (bit = 2)
 -			| load a       (00010010) 
 -			| b = 1 << bit (00000100)
 load a       (00010010)| -
 b = 1 << bit (00000001)| -
 a = a | b    (00010011)| -
...			| a = a | b    (00010110)
Actual result:   00010110
Expected result: 00010111	

3. Protection from double increment/decrement of counter of changed blocks
--------------------
### Potential threat
Losing changed blocks of changelog in case of extra decrements; 
Inability of freeing empty chunk in case of extra increments.

### Shared resource
struct bitmap_chunk {
	uint16_t changed_blocks;
	...
}

### Critical section
Setting the first or clearing the last bit of a block

### Security facility
The major bit of a block

### Example of race condition
code:
	if (!*i.block)
		i.chunk->changed_blocks++;
Assume initial value of block is 0

Sequence which causes race:
thread 1			| thread 2
 -				| test block, condition is true 
test block, condition is true   | -
chunk->changed_blocks++		| -
...				| -
				| chunk->changed_blocks++ 
Actual result: counter increased by two
Expected result: counter increased by one

