#include "blksnap-bio-submit.h"

struct blksnap_bio_ret {
	struct completion event;
	int error;
};

/* see blksnap-compat.h bio_endio block */
BIO_ENDIO_TYPE blksnap_bio_complete_passby(BIO_ENDIO_PARAMS)
{
	struct blksnap_bio_ret *ret = (struct blksnap_bio_ret *) bio->bi_private;

	ret->error = COMPAT_BIO_ERROR(bio);
	complete(&ret->event);

	return BIO_ENDIO_ERR;
}

BIO_ENDIO_TYPE blksnap_bio_complete_filtered(BIO_ENDIO_PARAMS)
{
	struct blksnap_bio_ret *ret = (struct blksnap_bio_ret *) bio->bi_private;

	ret->error = COMPAT_BIO_ERROR(bio);
	complete(&ret->event);

	return BIO_ENDIO_ERR;
}

/* @op - blksnap accepted operations:
 *	 BLKSNAP_READ	- read bio
 *	 BLKSNAP_WRITE	- write bio to be passed TO the filter
 *	 function
 *	 BLKSNAP_WRITE_FILTER_DISABLED - write bio to be passed BY the filter
 */
int blksnap_submit_bio_wait(struct bio *bio, int op)
{
	struct blksnap_bio_ret ret;

	/* setup bio and submit */
	if (op == BLKSNAP_READ)
		COMPAT_BIO_SET_OP(bio, READ, READ_SYNC);
	else
		COMPAT_BIO_SET_OP(bio, WRITE, WRITE_SYNC);

	init_completion(&ret.event);
	bio->bi_private = &ret;

	if (op == BLKSNAP_WRITE_FILTER_DISABLED)
		bio->bi_end_io = blksnap_bio_complete_passby;
	else
		bio->bi_end_io = blksnap_bio_complete_filtered;
#if HAVE_BIO_ASSOCIATE_BLKG
	bio_associate_blkg(bio);
#endif
	COMPAT_SUBMIT_BIO(bio);
	wait_for_completion(&ret.event);

	return ret.error;
}

