#ifndef _BLKSNAP_BITMAP_H
#define _BLKSNAP_BITMAP_H

/* This module describes operations with RAM representation of changelog */

/* Is this block a part of the current snapshot? This causes inclusion in
 * the changelog and allows pruning once the snapshot is complete. */
#define THIS_SNAP	0

/* Is this block a part of the next snapshot? This prevents tree pruning done
 * during snapshot end from removing this block from the tree. */
#define NEXT_SNAP	1

/* Is a COW operation in progress for this block?
 * Prevents two writes from racing. */
#define IN_FLIGHT	2

/* Is this block commited to the cl store? */
#define COMMITTED	3

/* Shows that current block is having cow information */
#define COW_DATA	4

/* The block has changed since the last snapshot
 * This bit prevents collisions of increment of counter of changed blocks
 * It can be replaced with a chunk spinlock if extra bit is needed */
#define CHANGED		7

#define BLOCK_STATE_MASK	(0xFF >> 1)

sector_t last_bitmap_block_addr(sector_t start_sector, sector_t nsectors);

int blksnap_blk_bitmap_create(struct blksnap_volume *volume);

void blksnap_blk_bitmap_free(struct blksnap_volume *volume);

int blksnap_blk_bitmap_reset(struct blksnap_volume *volume);

void blksnap_blk_bitmap_apply_start_snapshot(struct blksnap_volume *volume);

int blksnap_blk_bitmap_apply_stop_snapshot(struct blksnap_volume *volume,
					   bool merge);

int blksnap_blk_bitmap_write_changelog(struct blksnap_volume *volume);

void blksnap_show_changelog(struct blksnap_volume *volume);

sector_t blksnap_get_changelog_size(struct blksnap_volume *volume);

u32 blksnap_get_changelog_extents(struct blksnap_volume *volume,
				  struct blksnap_ioctl_disk_extent extents[],
				  u32 nextents,
				  sector_t offset);

#endif /* _BLKSNAP_BITMAP_H */
