#ifndef _BLKSNAP_CL_BUFFER_H_
#define _BLKSNAP_CL_BUFFER_H_

/* Structs definition */
struct blksnap_cl_stat {
	uint32_t allocated_pages;
	uint32_t pages_written;
	uint32_t overflow_count;
	uint32_t error_count;
	uint32_t saved_block_count;
};

/* It's not necessary to share buffer internal structure */
struct blksnap_cl_page;
struct blksnap_cl_buffer;

/* This callback should be called during flush to post page to disk */
typedef int (*writer_func)(sector_t *page_buffer, unsigned int len,
			   void *data);
/* Called in case if we have page ready to be written */
typedef int (*flush_page_func)(struct blksnap_cl_page *page, void *data);

/* Initialization functions */
struct blksnap_cl_buffer *blksnap_cl_buffer_init(
					 flush_page_func flush_callback,
					 void *flush_page_data,
					 writer_func write_callback,
					 void *writer_data);
int blksnap_cl_buffer_destroy(struct blksnap_cl_buffer *buffer);

/* This function add new block number. This function is overflow safe */
int blksnap_cl_buffer_add_block(struct blksnap_cl_buffer *buffer,
				sector_t addr);

/* Performing flush of known page to disk */
int blksnap_cl_page_flush(struct blksnap_cl_buffer *buffer,
			  struct blksnap_cl_page *page);

/* Return structure with statistic information */
int blksnap_cl_get_stat(struct blksnap_cl_buffer *buffer,
			struct blksnap_cl_stat *stats);

#endif /*_BLKSNAP_CL_BUFFER_H_ */
