#ifndef _BLKSNAP_CL_STORE_H
#define _BLKSNAP_CL_STORE_H

#include "blksnap-device.h"
#include "blksnap-bio.h"
#include "blksnap-extent.h"
#include "blksnap-cl-thread.h"

/* This module describes operations with ONDISK representation of changelog */

enum cl_store_status {
	CL_STORE_NOSTORE = 0,
	CL_STORE_LOADING,
	CL_STORE_MAPPED,
	CL_STORE_INVALID,
	CL_STORE_SHUTDOWN,
};

#define MAP_NOOPS (0x0)
#define MAP_RESET (0x1)
#define MAP_READ  (0x1 << 1)
#define MAP_WRITE (0x1 << 2)

/* {{{1 Structures */

struct pos_in_ext_map {
	struct blksnap_extent *ext;
	/* Internal position measured in sectors from ext->start */
	sector_t pos;
};

/* For reporting to dmesg about portion of blocks are written in changelog store
 * on stop_snapshot, detach and etc. */
struct cl_store_stat {
	uint64_t blocks;
	uint64_t added;
};

struct blksnap_cl_store {
	struct block_device *bdev;
	struct blksnap_extents extents;
	/* size and usage of on-disk storage, bytes */
	uint64_t size;
	uint64_t usage;
	struct pos_in_ext_map cur_pos;
	struct cl_thread *thread;
	struct cl_thread_opts opts;
	/* clstore status */
	atomic_t status;
	struct mutex lock;
	struct cl_store_stat stat;
};

int cl_store_map(struct blksnap_volume *volume,
		 struct blksnap_ioctl_map *map,
		 int operation);

int cl_store_commit_block(struct blksnap_volume *volume, sector_t addr,
			  bool stat);

int __cl_store_commit_block(struct blksnap_volume *volume, sector_t addr,
			    struct blksnap_bitmap_iter *index);

struct blksnap_cl_store *cl_store_init(struct block_device *bdev);

void cl_store_destroy(struct blksnap_cl_store *store);

int cl_store_reset(struct blksnap_cl_store *store);

int cl_store_status(struct blksnap_cl_store *store);

static inline bool blksnap_cl_store_valid(struct blksnap_cl_store *store)
{
	return (cl_store_status(store) == CL_STORE_MAPPED);
}

void cl_store_usage_add_flushed_blocks(struct blksnap_cl_store *store,
				       uint64_t bytes);

static inline void blksnap_cl_store_stat_reset(struct blksnap_cl_store *store)
{
	store->stat.blocks = store->stat.added = 0;
}

static inline void blksnap_cl_store_stat_show(struct blksnap_cl_store *store)
{
	bsmsg_bdev(store->bdev,
		   "Merged tree to the store. Added %llu blocks of %llu",
		   store->stat.added, store->stat.blocks);
}

#endif /* _BLKSNAP_CL_STORE_H */

