#ifndef _BLKSNAP_COMPAT_H
#define _BLKSNAP_COMPAT_H

#include <linux/blkdev.h>
#include <linux/version.h>
#include <linux/utsname.h>

#if defined __has_include
#  if __has_include (<linux/genhd.h>)
#    include <linux/genhd.h>
#  endif
#else
#  include <linux/genhd.h>
#endif

#include "blksnap-config.h"

#define MISSING_SG_GAPS		(LINUX_VERSION_CODE <  KERNEL_VERSION(3, 16, 0))

#define GENERATED_UTSRELEASE	(LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 33))
#if GENERATED_UTSRELEASE
#  include <generated/utsrelease.h>
#else
#  include <linux/utsrelease.h>
#endif

#define HAVE_EXPORT_H		(LINUX_VERSION_CODE >= KERNEL_VERSION(3, 2, 0))
#if HAVE_EXPORT_H
#  include <linux/export.h>
#endif

#define HAVE_CRYPTOHASH_H	(LINUX_VERSION_CODE < KERNEL_VERSION(5, 8, 0))
#if HAVE_CRYPTOHASH_H
#  include <linux/cryptohash.h>
#endif

#if HAVE_BLK_MQ_MAKE_REQUEST
#  include <linux/blk-mq.h>
#endif

#define HAVE_BLK_MQ_SUBMIT_BIO		(LINUX_VERSION_CODE >= KERNEL_VERSION(5, 9, 0))
#if HAVE_BLK_MQ_SUBMIT_BIO
#  include <linux/blk-mq.h>
#  define BLK_MQ_SUBMIT_BIO_EXPORTED	(LINUX_VERSION_CODE < KERNEL_VERSION(5, 9, 2))
#endif

#define HAVE_MAKE_REQUEST_FN		!HAVE_BLK_MQ_SUBMIT_BIO

#define HAVE_BDEV_STACK_LIMITS		(LINUX_VERSION_CODE < KERNEL_VERSION(5, 9, 0))

#define BDGET_DISK_EXPORTED		(LINUX_VERSION_CODE < KERNEL_VERSION(5, 11, 0))

#define HAVE_BIO_MAX_VECS		(LINUX_VERSION_CODE >= KERNEL_VERSION(5, 12, 0))

#define HAVE_BOUNCE_GFP		(LINUX_VERSION_CODE < KERNEL_VERSION(5, 13, 0))

#define BLK_ALLOC_QUEUE_EXPORTED	(LINUX_VERSION_CODE < KERNEL_VERSION(5, 14, 0))

#define GENDISK_HAS_OPEN_PARTITIONS	(LINUX_VERSION_CODE >= KERNEL_VERSION(5, 14, 0))

#define HAVE_PRINTK		(PRINTK_ADDR != 0x0)

#if HAVE_PRINTK
#  define COMPAT_SYS_CALL_ADDR_DISP	(long long)(((void *)printk) - (void *)PRINTK_ADDR)
#else
#  define COMPAT_SYS_CALL_ADDR_DISP	(long long)(((void *)_printk) - (void *)_PRINTK_ADDR)
#endif

#define BLK_ALLOC_QUEUE_2_ARGS		(LINUX_VERSION_CODE >= KERNEL_VERSION(5, 17, 0))

#define USE_BIO_KMALLOC			!BIO_ALLOC_4_ARGS

#ifndef MIN_NICE
#  define MIN_NICE -20
#endif

#ifndef BIO_MAX_SIZE	/* kernel >= 4.8 */
#  if HAVE_BIO_MAX_VECS
#	define BIO_MAX_SIZE BIO_MAX_VECS << PAGE_SHIFT
#  else
#	define BIO_MAX_SIZE BIO_MAX_PAGES << PAGE_SHIFT
#  endif
#endif

#ifndef __UNIQUE_ID
# define ___PASTE(a,b) a##b
# define __PASTE(a,b) ___PASTE(a,b)
# define __UNIQUE_ID(prefix) __PASTE(__PASTE(__UNIQUE_ID_, prefix), __COUNTER__)
#endif

#if !HAVE_BIO_LIST
struct bio_list {
	struct bio *head;
	struct bio *tail;
};

static inline int bio_list_empty(const struct bio_list *bl)
{
	return bl->head == NULL;
}

static inline void bio_list_init(struct bio_list *bl)
{
	bl->head = bl->tail = NULL;
}

static inline void bio_list_add(struct bio_list *bl, struct bio *bio)
{
	bio->bi_next = NULL;

	if (bl->tail)
		bl->tail->bi_next = bio;
	else
		bl->head = bio;

	bl->tail = bio;
}

static inline struct bio *bio_list_pop(struct bio_list *bl)
{
	struct bio *bio = bl->head;

	if (bio) {
		bl->head = bl->head->bi_next;
		if (!bl->head)
			bl->tail = NULL;

		bio->bi_next = NULL;
	}

	return bio;
}
#endif /* !HAVE_BIO_LIST */

#if !HAVE_BVEC_ITER
   typedef int compat_bvec_iter_t;
   typedef struct bio_vec * compat_bio_vec_t;
#  define COMPAT_BIO_SIZE(bio)		((bio)->bi_size)
#  define COMPAT_BIO_SECTOR(bio)	((bio)->bi_sector)
#  define COMPAT_BIO_IDX(bio)		((bio)->bi_idx)
#  define COMPAT_BIO_ITER_LEN(bio, iter) \
	  ((bio)->bi_io_vec[(iter)].bv_len)
#  define COMPAT_BIO_ITER_OFFSET(bio, iter) \
	  ((bio)->bi_io_vec[(iter)].bv_offset)
#  define COMPAT_BIO_ITER_PAGE(bio, iter) \
	  ((bio)->bi_io_vec[(iter)].bv_page)
#  define COMPAT_BIO_ITER_IDX(iter) (iter)
#  define COMPAT_BIO_BVEC_PAGE(bv) (bv->bv_page)
#  define COMPAT_BIO_BVEC_LEN(bv) (bv->bv_len)
#  define COMPAT_BIO_BVEC_OFFSET(bv) (bv->bv_offset)
#  define COMPAT_INC_BIO_REMAINING(bio) /* nothing */
#else /* HAVE_BVEC_ITER */
   typedef struct bvec_iter compat_bvec_iter_t;
   typedef struct bio_vec compat_bio_vec_t;
#  define COMPAT_BIO_SIZE(bio)		((bio)->bi_iter.bi_size)
#  define COMPAT_BIO_SECTOR(bio)	((bio)->bi_iter.bi_sector)
#  define COMPAT_BIO_IDX(bio)		((bio)->bi_iter.bi_idx)
#  define COMPAT_BIO_ITER_LEN(bio, iter) \
	  bio_iter_len((bio), (iter))
#  define COMPAT_BIO_ITER_OFFSET(bio, iter) \
	  bio_iter_offset((bio), (iter))
#  define COMPAT_BIO_ITER_PAGE(bio, iter) \
	  bio_iter_page((bio), (iter))
#  define COMPAT_BIO_ITER_IDX(iter) (iter.bi_idx)
#  define COMPAT_BIO_BVEC_PAGE(bv) (bv.bv_page)
#  define COMPAT_BIO_BVEC_LEN(bv) (bv.bv_len)
#  define COMPAT_BIO_BVEC_OFFSET(bv) (bv.bv_offset)
  /* In kernels >= 3.14 there is a mechanism to keep splitted bios
   * uptodate using bi_remaining. Due to kernel bug this variable uses
   * for all bio requests not for splitted only. This bug is fixed
   * in kernel >= 4.2. Between these version it's required to increment
   * bi_remaining in our own bi_end_io() if we plan to use this bio request
   * again with other parameters.
   */
#  define OLD_BIO_REMAINING	(LINUX_VERSION_CODE < KERNEL_VERSION(4, 2, 0))
#  if OLD_BIO_REMAINING
#    define COMPAT_INC_BIO_REMAINING(bio) atomic_inc(&(bio)->bi_remaining)
#  else
#    define COMPAT_INC_BIO_REMAINING(bio) /* nothing */
#  endif /* OLD_BIO_REMAINING */
#endif /* HAVE_BVEC_ITER */

#if !HAVE_BIO_HAS_DATA
static inline bool bio_has_data(struct bio *bio)
{
	if (bio && bio->bi_vcnt)
		return true;

	return false;
}
#endif /* !HAVE_BIO_HAS_DATA */

#if !defined(DISK_NAME_LEN)
#  define DISK_NAME_LEN 32
#endif

#if !HAVE_UTSNAME
static inline struct new_utsname *utsname(void)
{
	return &system_utsname;
}
#endif

#if !defined(FLEX_ARRAY_FREE)
#  define FLEX_ARRAY_FREE 0x6c
#endif

#if BLKDEV_GET_2_ARGS
#  define COMPAT_BLKDEV_GET(bdev, mode)		blkdev_get(bdev, mode);
#elif BLKDEV_GET_3_ARGS
#  define COMPAT_BLKDEV_GET(bdev, mode)		blkdev_get(bdev, mode, 0);
#else
#if !HAVE_BLKDEV_GET_BY_PATH
#  error Cannot determine a type of blkdev_get() function. Contact developers please
#endif
#endif

#if BLKDEV_PUT_1_ARG
#  define COMPAT_BLKDEV_PUT(bdev, mode)		blkdev_put(bdev);
#elif BLKDEV_PUT_2_ARGS
#  define COMPAT_BLKDEV_PUT(bdev, mode)		blkdev_put(bdev, mode);
#else
#  error Cannot determine a type of blkdev_put() function. Contact developers please
#endif

static inline struct block_device *blksnap_lookup_bdev_compat(const char* path)
{
#if LOOKUP_BDEV_2_ARGS_RET_INT
	dev_t dev;
	int error;

	struct block_device *(*blksnap_blkdev_get_no_open)(dev_t) = (struct block_device *(*)(dev_t)) (BLKDEV_GET_NO_OPEN_ADDR + COMPAT_SYS_CALL_ADDR_DISP);

	error = lookup_bdev(path, &dev);
	if (error)
		return NULL;

	return blksnap_blkdev_get_no_open(dev);
#elif LOOKUP_BDEV_2_ARGS
	return lookup_bdev(path, 0);
#elif LOOKUP_BDEV_1_ARG
	return lookup_bdev(path);
#else
#  error Cannot determine a type of lookup_bdev() function. Contact developers please
	return NULL;
#endif
}

#if BIO_HAS_BI_STATUS
static const int blksnap_status_errors[] = {
	[BLK_STS_OK]			= 0,
	[BLK_STS_NOTSUPP]		= -EOPNOTSUPP,
	[BLK_STS_TIMEOUT]		= -ETIMEDOUT,
	[BLK_STS_NOSPC]			= -ENOSPC,
	[BLK_STS_TRANSPORT]		= -ENOLINK,
	[BLK_STS_TARGET]		= -EREMOTEIO,
	[BLK_STS_NEXUS]			= -EBADE,
	[BLK_STS_MEDIUM]		= -ENODATA,
	[BLK_STS_PROTECTION]		= -EILSEQ,
	[BLK_STS_RESOURCE]		= -ENOMEM,
	[BLK_STS_AGAIN]			= -EAGAIN,
	[BLK_STS_DM_REQUEUE]		= -EREMCHG,
	[BLK_STS_IOERR]			= -EIO
};

static inline blk_status_t blksnap_errno_to_status(int errno)
{
	int status;

	for (status = 0;
	     status < ARRAY_SIZE(blksnap_status_errors);
	     status++) {
		if (blksnap_status_errors[status] == errno)
			return (__force blk_status_t) status;
	}

	/* errno is not defined in status array */
	return BLK_STS_IOERR;
}

static inline int blksnap_status_to_errno(blk_status_t status)
{
	if ((int) status >= ARRAY_SIZE(blksnap_status_errors))
		return -EIO;

	return blksnap_status_errors[(int) status];
}
#endif

/* bio error compatibility:
 * - the oldest method is to check if bio size is ! 0;
 * - better method is to check if bio flags are validl
 * - another method is to store error in bi_error bio member;
 * - actual method is to store error in blk_status_t bi_status bio member
 */
#if BIO_HAS_BI_STATUS
#  define COMPAT_BIO_ERROR(bio)		blksnap_status_to_errno(bio->bi_status)
#elif BIO_HAS_BI_ERROR
#  define COMPAT_BIO_ERROR(bio)		(bio->bi_error)
#elif BIO_ENDIO_3_ARGS
#  define COMPAT_BIO_ERROR(bio)		(COMPAT_BIO_SIZE(bio) != 0)
#else
#  define COMPAT_BIO_ERROR(bio)		(!test_bit(BIO_UPTODATE, &bio->bi_flags))
#endif

/* bio_endio compatibilities */
#if BIO_ENDIO_3_ARGS
static inline void blksnap_bio_endio(struct bio *bio, int err)
{
	bio_endio(bio, bio->bi_size, err);
}
#  define BIO_ENDIO_TYPE		int
#  define BIO_ENDIO_ERR			err
#  define BIO_ENDIO_ERR_VAL		err
#  define BIO_ENDIO_PARAMS		struct bio *bio, \
					unsigned int bytes_done, \
					int err
#elif BIO_ENDIO_1_ARG
static inline void blksnap_bio_endio(struct bio *bio, int err)
{
#  if BIO_HAS_BI_STATUS
	bio->bi_status = blksnap_errno_to_status(err);
#  else
	bio->bi_error = err;
#endif
	bio_endio(bio);
}
#  define BIO_ENDIO_TYPE		void
#  define BIO_ENDIO_ERR
#  define BIO_ENDIO_ERR_VAL		0
#  define BIO_ENDIO_PARAMS		struct bio *bio
#elif BIO_ENDIO_2_ARGS
static inline void blksnap_bio_endio(struct bio *bio, int err)
{
	bio_endio(bio, err);
}
#  define BIO_ENDIO_TYPE		void
#  define BIO_ENDIO_ERR
#  define BIO_ENDIO_ERR_VAL		err
#  define BIO_ENDIO_PARAMS		struct bio *bio, int err
#else
#  error Cannot determine a type of bio_endio() function. Contact developers please
#endif

#if BIO_SPLIT_2_ARGS
#  define COMPAT_BIO_SPLIT(bio, first_sectors) \
	bio_split(bio, first_sectors)
#elif BIO_SPLIT_3_ARGS
#  define COMPAT_BIO_SPLIT(bio, first_sectors)	\
	bio_split(bio, bio_split_pool, first_sectors)
#elif BIO_SPLIT_4_ARGS
#  define COMPAT_BIO_SPLIT(bio, first_sectors) \
	bio_split(bio, first_sectors, GFP_NOIO, bio->bi_pool)
#else
#  error Cannot determine a type of bio_split() function. Contact developers please
#endif

#if SUBMIT_BIO_2_ARGS
#  define COMPAT_SUBMIT_BIO(bio) \
	submit_bio(bio->bi_rw, bio)
#else
#  define COMPAT_SUBMIT_BIO(bio) \
	submit_bio(bio)
#endif

#if BIO_HAS_BI_RW
#  define COMPAT_BIO_SET_OP(bio, OP, OP_FLAGS)	\
	bio->bi_rw = OP | OP_FLAGS
#elif HAVE_BIO_SET_OP_ATTRS
#  define COMPAT_BIO_SET_OP(bio, OP, OP_FLAGS)	\
	bio_set_op_attrs(bio, OP, OP_FLAGS)
#else
#  define COMPAT_BIO_SET_OP(bio, OP, OP_FLAGS)	\
	bio->bi_opf = OP | OP_FLAGS
#endif

#if !HAVE_BLKDEV_GET_BY_PATH
static inline struct block_device *blkdev_get_by_path(const char *path,
						      fmode_t mode,
						      void *holder)
{
	struct block_device *bdev = lookup_bdev(path);
	int error;
	if (IS_ERR(bdev))
		return bdev;
	error = COMPAT_BLKDEV_GET(bdev, mode);
	return error ? ERR_PTR(error) : bdev;
}
#endif

#ifndef bio_for_each_segment_all
#  define bio_for_each_segment_all(bvl, bio, i) \
		for (i = 0, bvl = (bio)->bi_io_vec; i < (bio)->bi_vcnt; i++, bvl++)
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(5, 1, 0)
#  define blksnap_bio_for_each_segment_all(bvl, bio) \
		int i; \
		bio_for_each_segment_all(bvl, bio, i)
#elif LINUX_VERSION_CODE < KERNEL_VERSION(5, 2, 0)
#  define blksnap_bio_for_each_segment_all(bvl, bio) \
		int i; \
		struct bvec_iter_all iter; \
		bio_for_each_segment_all(bvl, bio, i, iter)
#else
#  define blksnap_bio_for_each_segment_all(bvl, bio) \
		struct bvec_iter_all iter; \
		bio_for_each_segment_all(bvl, bio, iter)
#endif

#if !HAVE_BIO_FREE_PAGES
static inline void bio_free_pages(struct bio *bio)
{
	struct bio_vec *bvec;

	blksnap_bio_for_each_segment_all(bvec, bio)
		__free_page(bvec->bv_page);
}
#endif

#if HAVE_MAKE_REQUEST_FN
static inline struct request_queue *blksnap_alloc_queue(make_request_fn make_request)
{
	struct request_queue *ret = NULL;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(5, 7, 0))
	ret = blk_alloc_queue(make_request, NUMA_NO_NODE);
#else
	ret = blk_alloc_queue(GFP_KERNEL);
	if (!ret) {
		return NULL;
	}

	blk_queue_make_request(ret, make_request);
#endif
	return ret;
}
#else
static inline struct request_queue *blksnap_alloc_queue(void)
{
	struct request_queue *ret = NULL;
#if BLK_ALLOC_QUEUE_EXPORTED
	ret = blk_alloc_queue(NUMA_NO_NODE);
#else
#  if BLK_ALLOC_QUEUE_2_ARGS
	struct request_queue* (*blksnap_blk_alloc_queue)(int, bool) = (struct request_queue* (*)(int, bool)) (BLK_ALLOC_QUEUE_ADDR + COMPAT_SYS_CALL_ADDR_DISP);
	ret = blksnap_blk_alloc_queue(NUMA_NO_NODE, false);
#  else
	struct request_queue* (*blksnap_blk_alloc_queue)(int) = (struct request_queue* (*)(int)) (BLK_ALLOC_QUEUE_ADDR + COMPAT_SYS_CALL_ADDR_DISP);
	ret = blksnap_blk_alloc_queue(NUMA_NO_NODE);
#  endif
#endif

	return ret;
}
#endif

#if !HAVE_BLK_QUEUE_MAX_HW_SECTORS
#  define blk_queue_max_hw_sectors(q, max_sectors) \
	blk_queue_max_sectors(q, max_sectors)
#endif

#if !HAVE_BLK_QUEUE_MAX_SEGMENTS
#  define blk_queue_max_segments(q, max_segments) \
	blk_queue_max_hw_segments(q, max_segments)
#endif

static inline void blksnap_queue_set_flags(struct request_queue *q)
{
#if MISSING_SG_GAPS
	/* Older kernels didn't merge vectors */
	/* nothing to do */
#elif HAVE_BLK_QUEUE_VIRT_BOUNDARY
	/* Since 4.3 kernel have changed code to detect if requests are
	 * mergeable, it's required to use different method to prevent merging
	 * for our snapshot device
	 */
	blk_queue_virt_boundary(q, PAGE_SIZE - 1);
#else
	/* This flag MUST be set for queue that could serve bio_split()
	 * requests for kernels >= 3.18 to prevent a crash in blk_rq_map_sg()
	 * See D-26754
	 */
	queue_flag_set_unlocked(QUEUE_FLAG_SG_GAPS, q);
#endif
}

#if !HAVE_BLK_SET_STACKING_LIMITS
#  define blk_set_stacking_limits(queuelimits) \
	blk_set_default_limits(queuelimits)
#endif

#if KMAP_ATOMIC_2_ARGS
#  define COMPAT_KMAP_ATOMIC(p)		kmap_atomic(p, KM_USER0)
#  define COMPAT_KUNMAP_ATOMIC(a)	kunmap_atomic(a, KM_USER0)
#elif KMAP_ATOMIC_1_ARG
#  define COMPAT_KMAP_ATOMIC(p)		kmap_atomic(p)
#  define COMPAT_KUNMAP_ATOMIC(a)	kunmap_atomic(a)
#else
#  error Cannot determine a type of kmap_atomic() function. Contact developers please
#endif

#if MAKE_REQUEST_INT
#  define MRF_TYPE		int
#  define MRF_RET_VAR		ret
#  define MRF_LET_RET_VAR	ret =
#  define MRF_DEF_RET		0
#  define MRF_RET_VAR_INIT	MRF_TYPE ret = MRF_DEF_RET
#elif MAKE_REQUEST_BLKQCT
#  define MRF_TYPE		blk_qc_t
#  define MRF_RET_VAR		ret
#  define MRF_LET_RET_VAR	ret =
#  define MRF_DEF_RET		BLK_QC_T_NONE
#  define MRF_RET_VAR_INIT	MRF_TYPE ret = MRF_DEF_RET
#elif MAKE_REQUEST_VOID
#  define MRF_TYPE		void
#  define MRF_RET_VAR
#  define MRF_LET_RET_VAR
#  define MRF_DEF_RET
#  define MRF_RET_VAR_INIT
#elif HAVE_BLK_MQ_SUBMIT_BIO
#  if BLK_MQ_SUBMIT_BIO_VOID
#    define MRF_TYPE		void
#    define MRF_RET_VAR
#    define MRF_LET_RET_VAR
#    define MRF_DEF_RET
#    define MRF_RET_VAR_INIT
#  else
#    define MRF_TYPE		blk_qc_t
#    define MRF_RET_VAR 	ret
#    define MRF_LET_RET_VAR	ret =
#    define MRF_DEF_RET 	BLK_QC_T_NONE
#    define MRF_RET_VAR_INIT	MRF_TYPE ret = MRF_DEF_RET
#  endif
#else
#  error Cannot determine a type of make_request() function. Contact developers please
#endif

#if HAVE_MAKE_REQUEST_FN
static inline MRF_TYPE blksnap_make_request_compat(make_request_fn *make_request, struct request_queue *queue, struct bio *bio)
{
	MRF_RET_VAR_INIT;

	if (make_request != NULL) {
		MRF_LET_RET_VAR make_request(queue, bio);
	}
	else {
#if HAVE_BLK_MQ_MAKE_REQUEST
		MRF_LET_RET_VAR blk_mq_make_request(queue, bio);
#endif
	}

	return MRF_RET_VAR;
}
#else
static inline MRF_TYPE blksnap_submit_bio_compat(struct bio *bio)
{
	MRF_RET_VAR_INIT;

#if BLK_MQ_SUBMIT_BIO_EXPORTED
	MRF_LET_RET_VAR blk_mq_submit_bio(bio);
#else
	MRF_TYPE (*blksnap_blk_mq_submit_bio)(struct bio *) = (MRF_TYPE (*)(struct bio *)) (BLK_MQ_SUBMIT_BIO_ADDR + COMPAT_SYS_CALL_ADDR_DISP);
	MRF_LET_RET_VAR blksnap_blk_mq_submit_bio(bio);
#endif
	return MRF_RET_VAR;
}
#endif

#if ! HAVE_NOOP_LLSEEK
static inline loff_t noop_llseek(struct file *file, loff_t offset, int origin)
{
	return file->f_pos;
}
#endif

#if INT_THAW_BDEV
#  define THAW_VOL_RET_VAR	ret
#  define THAW_VOL_RET		int ret = 0
#  define THAW_VOL_RET_LET	ret =
#else /* void thaw_bdev() */
#  define THAW_VOL_RET_VAR	0
#  define THAW_VOL_RET
#  define THAW_VOL_RET_LET
#endif

#if BDEV_OPEN_RELEASE_INODE
#  define	RELEASE_TYPE		int
#  define	RELEASE_RET		0
#  define	RELEASE_PARAMS		struct inode *inode, struct file *file
#  define	RELEASE_PRIVATE_DATA	inode->i_bdev->bd_disk->private_data
#  define	OPEN_PARAMS		struct inode *inode, struct file *file
#  define	OPEN_PRIVATE_DATA	RELEASE_PRIVATE_DATA
#  define	OPEN_MODE_INIT		fmode_t mode = file->f_mode
#else
#  define	RELEASE_PARAMS		struct gendisk *disk, fmode_t mode
#  define	RELEASE_PRIVATE_DATA	disk->private_data
#  define	OPEN_PARAMS		struct block_device *bdev, fmode_t mode
#  define	OPEN_PRIVATE_DATA	bdev->bd_disk->private_data
#  define	OPEN_MODE_INIT
#  if BDEV_OPS_RELEASE_INT
#    define	RELEASE_TYPE		int
#    define	RELEASE_RET		0
#  else
#    define	RELEASE_TYPE		void
#    define	RELEASE_RET
#  endif
#endif

#ifndef READ_SYNC
#  define READ_SYNC		(READ | REQ_SYNC)
#endif

#ifndef WRITE_SYNC
#  define WRITE_SYNC		(WRITE | REQ_SYNC)
#endif

#ifndef SHA_MESSAGE_BYTES
#  define SHA_MESSAGE_BYTES (512 / 8)
#endif

#ifndef SHA_DIGEST_WORDS
#  define SHA_DIGEST_WORDS 5
#endif

#ifndef SHA_WORKSPACE_WORDS
#  define SHA_WORKSPACE_WORDS 16
#endif

#if HAVE_SHA_INIT
static inline void blksnap_sha_init(__u32 *buf)
{
	sha_init(buf);
}
#else
static inline void blksnap_sha_init(__u32 *buf)
{
	buf[0] = 0x67452301;
	buf[1] = 0xefcdab89;
	buf[2] = 0x98badcfe;
	buf[3] = 0x10325476;
	buf[4] = 0xc3d2e1f0;
}
#endif /* HAVE_SHA_INIT */

#if HAVE_SHA_TRANSFORM
static inline void blksnap_sha_transform(__u32 *digest, const char *data, __u32 *array)
{
	sha_transform(digest, data, array);
}
#else
#include <asm/unaligned.h>

#ifdef CONFIG_X86
  #define setW(x, val) (*(volatile __u32 *)&W(x) = (val))
#elif defined(CONFIG_ARM)
  #define setW(x, val) do { W(x) = (val); __asm__("":::"memory"); } while (0)
#else
  #define setW(x, val) (W(x) = (val))
#endif

#define W(x) (array[(x)&15])

#define SHA_SRC(t) get_unaligned_be32((__u32 *)data + t)
#define SHA_MIX(t) rol32(W(t+13) ^ W(t+8) ^ W(t+2) ^ W(t), 1)

#define SHA_ROUND(t, input, fn, constant, A, B, C, D, E) do { \
	__u32 TEMP = input(t); setW(t, TEMP); \
	E += TEMP + rol32(A,5) + (fn) + (constant); \
	B = ror32(B, 2); } while (0)

#define T_0_15(t, A, B, C, D, E)  SHA_ROUND(t, SHA_SRC, (((C^D)&B)^D) , 0x5a827999, A, B, C, D, E )
#define T_16_19(t, A, B, C, D, E) SHA_ROUND(t, SHA_MIX, (((C^D)&B)^D) , 0x5a827999, A, B, C, D, E )
#define T_20_39(t, A, B, C, D, E) SHA_ROUND(t, SHA_MIX, (B^C^D) , 0x6ed9eba1, A, B, C, D, E )
#define T_40_59(t, A, B, C, D, E) SHA_ROUND(t, SHA_MIX, ((B&C)+(D&(B^C))) , 0x8f1bbcdc, A, B, C, D, E )
#define T_60_79(t, A, B, C, D, E) SHA_ROUND(t, SHA_MIX, (B^C^D) ,  0xca62c1d6, A, B, C, D, E )

static inline void blksnap_sha_transform(__u32 *digest, const char *data, __u32 *array)
{
	__u32 A, B, C, D, E;

	A = digest[0];
	B = digest[1];
	C = digest[2];
	D = digest[3];
	E = digest[4];

	T_0_15( 0, A, B, C, D, E);
	T_0_15( 1, E, A, B, C, D);
	T_0_15( 2, D, E, A, B, C);
	T_0_15( 3, C, D, E, A, B);
	T_0_15( 4, B, C, D, E, A);
	T_0_15( 5, A, B, C, D, E);
	T_0_15( 6, E, A, B, C, D);
	T_0_15( 7, D, E, A, B, C);
	T_0_15( 8, C, D, E, A, B);
	T_0_15( 9, B, C, D, E, A);
	T_0_15(10, A, B, C, D, E);
	T_0_15(11, E, A, B, C, D);
	T_0_15(12, D, E, A, B, C);
	T_0_15(13, C, D, E, A, B);
	T_0_15(14, B, C, D, E, A);
	T_0_15(15, A, B, C, D, E);

	T_16_19(16, E, A, B, C, D);
	T_16_19(17, D, E, A, B, C);
	T_16_19(18, C, D, E, A, B);
	T_16_19(19, B, C, D, E, A);

	T_20_39(20, A, B, C, D, E);
	T_20_39(21, E, A, B, C, D);
	T_20_39(22, D, E, A, B, C);
	T_20_39(23, C, D, E, A, B);
	T_20_39(24, B, C, D, E, A);
	T_20_39(25, A, B, C, D, E);
	T_20_39(26, E, A, B, C, D);
	T_20_39(27, D, E, A, B, C);
	T_20_39(28, C, D, E, A, B);
	T_20_39(29, B, C, D, E, A);
	T_20_39(30, A, B, C, D, E);
	T_20_39(31, E, A, B, C, D);
	T_20_39(32, D, E, A, B, C);
	T_20_39(33, C, D, E, A, B);
	T_20_39(34, B, C, D, E, A);
	T_20_39(35, A, B, C, D, E);
	T_20_39(36, E, A, B, C, D);
	T_20_39(37, D, E, A, B, C);
	T_20_39(38, C, D, E, A, B);
	T_20_39(39, B, C, D, E, A);

	T_40_59(40, A, B, C, D, E);
	T_40_59(41, E, A, B, C, D);
	T_40_59(42, D, E, A, B, C);
	T_40_59(43, C, D, E, A, B);
	T_40_59(44, B, C, D, E, A);
	T_40_59(45, A, B, C, D, E);
	T_40_59(46, E, A, B, C, D);
	T_40_59(47, D, E, A, B, C);
	T_40_59(48, C, D, E, A, B);
	T_40_59(49, B, C, D, E, A);
	T_40_59(50, A, B, C, D, E);
	T_40_59(51, E, A, B, C, D);
	T_40_59(52, D, E, A, B, C);
	T_40_59(53, C, D, E, A, B);
	T_40_59(54, B, C, D, E, A);
	T_40_59(55, A, B, C, D, E);
	T_40_59(56, E, A, B, C, D);
	T_40_59(57, D, E, A, B, C);
	T_40_59(58, C, D, E, A, B);
	T_40_59(59, B, C, D, E, A);

	T_60_79(60, A, B, C, D, E);
	T_60_79(61, E, A, B, C, D);
	T_60_79(62, D, E, A, B, C);
	T_60_79(63, C, D, E, A, B);
	T_60_79(64, B, C, D, E, A);
	T_60_79(65, A, B, C, D, E);
	T_60_79(66, E, A, B, C, D);
	T_60_79(67, D, E, A, B, C);
	T_60_79(68, C, D, E, A, B);
	T_60_79(69, B, C, D, E, A);
	T_60_79(70, A, B, C, D, E);
	T_60_79(71, E, A, B, C, D);
	T_60_79(72, D, E, A, B, C);
	T_60_79(73, C, D, E, A, B);
	T_60_79(74, B, C, D, E, A);
	T_60_79(75, A, B, C, D, E);
	T_60_79(76, E, A, B, C, D);
	T_60_79(77, D, E, A, B, C);
	T_60_79(78, C, D, E, A, B);
	T_60_79(79, B, C, D, E, A);

	digest[0] += A;
	digest[1] += B;
	digest[2] += C;
	digest[3] += D;
	digest[4] += E;
}
#endif /* HAVE_SHA_TRANSFORM */

#if BIO_HAS_BI_DISK
#  define COMPAT_BIO_SET_BDEV(bio, bdev)	bio_set_dev(bio, bdev)
#  define COMPAT_BIO_GET_QUEUE(bio)		((bio)->bi_disk->queue)
#else
#  if HAVE_BIO_SET_DEV
#	define COMPAT_BIO_SET_BDEV(bio, bdev)	bio_set_dev(bio, bdev)
#  else
#	define COMPAT_BIO_SET_BDEV(bio, bdev)	(bio->bi_bdev = bdev)
#  endif
#  define COMPAT_BIO_GET_QUEUE(bio)		bdev_get_queue((bio)->bi_bdev)
#endif /* BIO_HAS_BI_DISK */

#endif /* _BLKSNAP_COMPAT_H */
