#ifndef _BLKSNAP_COW_TREE_H
#define _BLKSNAP_COW_TREE_H

#include <linux/types.h>

#include <blksnap.h>
#include "blksnap-device.h"

/* The basic unit of capture. This represents both a portion of disk data and
 * a node within the cow block tree. */
struct blksnap_cow_block {
	/* Blocksize aligned sector address. */
	sector_t addr;

	/* Block tree membership. See the blksnap_device structure. */
	struct rb_node rb_node;

	/* Members of this structure are protected by this lock */
	rwlock_t lock;

	/* Data is committed to disk/memory */
	bool committed;

	/* Offset in the extent map, if this page is stored on disk */
	sector_t extent_offset;

	/* Page data if this page is stored in memory */
	struct page *page;
};

struct blksnap_cow_block *cow_tree_search(struct blksnap_volume *volume,
					  sector_t addr);

struct blksnap_cow_block *__cow_tree_search(struct rb_root *root,
					    sector_t addr);

struct blksnap_cow_block *cow_tree_insert(struct blksnap_volume *volume,
					  sector_t addr);

int cow_tree_empty(struct blksnap_volume *volume);

void cow_block_sanity_check(struct blksnap_volume *volume,
			    struct blksnap_cow_block *blk,
			    bool inserted);

#endif /* _BLKSNAP_COW_TREE_H */
