#ifndef _BLKSNAP_DATA_H
#define _BLKSNAP_DATA_H

#include <linux/fs.h>
#include <linux/types.h>

#include <blksnap.h>
#include "blksnap-compat.h"
#include "blksnap-device.h"
#include "blksnap-cow-tree.h"

/* {{{1 Store Structure */
struct blksnap_data_store {

	struct block_device *bdev;

	/* Physical extents that make up the on-disk store */
	struct blksnap_extents extents;

	/* Exclusive slot reservations */
	struct mutex reservation;

	/* In-memory pages */
	u64 npages_mem_now;
	u64 npages_mem_max;

	/* On-disk pages */
	u64 npages_disk_now;
	u64 npages_disk_max;

	/* Indicate the store is mapped completely */
	bool valid;
};

/* {{{1 Store-Wide operations */
struct blksnap_data_store *blksnap_data_store_alloc(struct block_device *bdev);
void blksnap_data_store_free(struct blksnap_data_store *store);
bool blksnap_data_store_is_valid(struct blksnap_data_store *store);
void blksnap_data_store_reset(struct blksnap_data_store *store);
int  blksnap_data_store_unmap(struct blksnap_data_store *store);
int  blksnap_data_store_map (struct blksnap_data_store *store,
			     struct blksnap_ioctl_map *map);

/* {{{1 Individual CoW-Block operations */
int blksnap_data_cow_alloc(struct blksnap_data_store *store,
			   struct blksnap_cow_block *cow_blk);
bool blksnap_data_cow_has_data(struct blksnap_cow_block *cow_blk);
void blksnap_data_cow_disk_commit(struct blksnap_cow_block *cow_blk);
void blksnap_data_cow_free(struct blksnap_cow_block *cow_blk);
void blksnap_data_cow_show(struct blksnap_cow_block *cow_blk);
bool blksnap_data_cow_put(struct blksnap_data_store *store,
			  struct blksnap_cow_block *cow_blk,
			  struct bio *bio, int *err);
bool blksnap_data_cow_get(struct blksnap_data_store *store,
			  struct blksnap_cow_block *cow_blk,
			  struct bio *bio, int *err);

#endif /* _BLKSNAP_DATA_H */
