#ifndef _BLKSNAP_HASH_H
#define _BLKSNAP_HASH_H

#include "blksnap-compat.h"

#define BLKSNAP_HASH_DIGEST_WORDS SHA_DIGEST_WORDS
#define BLKSNAP_HASH_DIGEST_BYTES (BLKSNAP_HASH_DIGEST_WORDS * sizeof(u32))

#define BLKSNAP_HASH_MSG_BYTES    SHA_MESSAGE_BYTES
#define BLKSNAP_HASH_MSG_WORDS    (BLKSNAP_HASH_MSG_BYTES / (sizeof(u32))

#define BLKSNAP_HASH_WS_WORDS     SHA_WORKSPACE_WORDS
#define BLKSNAP_HASH_WS_BYTES     (SHA_WORKSPACE_WORDS * sizeof(u32))

/* Padding in sha1 block to leave 8 bytes to store length */
#define BLKSNAP_PADDING_BYTES (BLKSNAP_HASH_MSG_BYTES - sizeof(uint64_t))

/*
 * Calculating hash from given data.
 * Return value:
 *     0 - on success, error code otherwise
 */
uint64_t blksnap_get_data_hash_size(uint64_t data_size);
int blksnap_calculate_hash(u8 *data_hash, uint64_t data_size, u32 *out);

struct blksnap_hasher {
	uint32_t digest[BLKSNAP_HASH_DIGEST_WORDS];
	uint32_t ws[BLKSNAP_HASH_WS_WORDS];
	uint8_t tail_data[BLKSNAP_HASH_MSG_BYTES];
};

/*
 * Calculating hash from in-flight data.
 * total_data_size and data_size should be multiple to BLKSNAP_HASH_MSG_BYTES(64 bytes)
 */
void blksnap_hasher_init(struct blksnap_hasher *hasher, uint64_t total_data_size);
void blksnap_hasher_update(struct blksnap_hasher *hasher, uint8_t *data, uint64_t data_size);
void blksnap_hasher_finalize(struct blksnap_hasher *hasher, u32 *out);

#endif /* _BLKSNAP_HASH_H */
