#ifndef _BLKSNAP_VOLUME_H
#define _BLKSNAP_VOLUME_H

#include <linux/fs.h>
#include <linux/types.h>

#include "blksnap-shared.h"
#include "blksnap-device.h"
#include "blksnap-device-utils.h"

#define BLKSNAP_VOLUME_MISSING "VOLUME POINTER MISSING!"

/* List of volumes. These are block devices that the user maniplates through
 * the ioctl interface. */
extern struct list_head blksnap_volumes;

struct blksnap_volume *lookup_volume_by_bdev(struct block_device *bdev);
struct blksnap_volume *lookup_volume_by_path(const char* path);
struct blksnap_volume *lookup_volume_by_addr(struct blksnap_device *bd,
					     sector_t addr);

bool volume_contains(struct blksnap_volume *volume, sector_t addr);

#if THAW_BDEV_2_ARGS
struct super_block *freeze_volume(struct blksnap_volume *volume);
#else
struct blksnap_volume *freeze_volume(struct blksnap_volume *volume);
#endif
int thaw_volume(struct blksnap_volume *volume);

static inline bool snapshot_is_incremental(struct blksnap_volume *volume)
{
	return test_bit(VOLUME_SNAP_INCR, &volume->flags);
}

static inline bool volume_has_snapshot(struct blksnap_volume *volume)
{
	return test_bit(VOLUME_SNAP_ON, &volume->flags);
}

static inline bool volume_is_grabbing_map(struct blksnap_volume *volume)
{
	return test_bit(VOLUME_GRAB_MAP, &volume->flags);
}

static inline void snapshot_error_reset(struct blksnap_volume *volume)
{
	atomic_set(&volume->snapshot_errors, 0);
}

static inline void snapshot_error(struct blksnap_volume *volume)
{
	atomic_inc(&volume->snapshot_errors);
}

static inline bool snapshot_has_errors(struct blksnap_volume *volume)
{
	return atomic_read(&volume->snapshot_errors) > 0;
}

static inline sector_t volume_capacity(struct blksnap_volume *volume)
{
	return blksnap_get_nr_sects(volume->bdev);
}

static inline const char *volume_name(const struct blksnap_volume *volume)
{
	if (volume)
		return volume->name[0] ? volume->name : UNKNOWN_NAME;
	else
		return BLKSNAP_VOLUME_MISSING;
}

/* Convert offset from device related to volume related */
static inline sector_t volume_relative_address(struct block_device *device,
					       sector_t addr)
{
	return (addr - get_start_sect(device));
}

#endif /* _BLKSNAP_VOLUME_H */
