#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/bio.h>
#include <linux/blkdev.h>

#include "blksnap.h"
#include "blksnap-bio-queue-share.h"
#include "blksnap-bio-queue.h"
#include "blksnap-hash.h"
#include "blksnap-device.h"
#include "blksnap-compat.h"
#include "blksnap-shared.h"
#include "blksnap-write-queue.h"
#include "blksnap-device-utils.h"
#include "blksnap-volume.h"
#include "blksnap-bitmap.h"
#include "blksnap-block.h"
#include "blksnap-data.h"
#include "blksnap-cow-tree.h"
#include "blksnap-snapshot.h"

#define WRITE_BIO_THREAD_NAME	"write-bio"

/* callback to process original write bios
 * @param: bq - bio queue
 */
static void write_flush(struct blksnap_bio_queue *bq)
{
	unsigned long flags;
	struct blksnap_volume *volume = bq->volume;
	struct bio *bio;

	spin_lock_irqsave(&bq->lock, flags);
	bio = bio_list_pop(&bq->bios);
	spin_unlock_irqrestore(&bq->lock, flags);

	copy_write_bio(volume, bio);
}

unsigned int blksnap_write_bio_kthread_init(struct blksnap_volume *volume) {
	return blksnap_bio_kthread_init(volume, &volume->write_queue,
					WRITE_BIO_THREAD_NAME,
					write_flush);
}

