#!/bin/bash
TEST_DIR=$(cd $(dirname $0); pwd)
SYMBOLS_TO_GET_ADDR_FILE="$TEST_DIR/symbols-to-get-addr"
OUTPUT_FILE=$TEST_DIR/../blksnap-config.h
MAKE_LOG=$TEST_DIR/make.log

echo "KERNELRELEASE=$KERNELRELEASE"
SYSTEM_MAP_FILE="/lib/modules/${KERNELRELEASE}/System.map"

# Trying to use /boot location
[ ! -f "$SYSTEM_MAP_FILE" ] && SYSTEM_MAP_FILE="/boot/System.map-${KERNELRELEASE}"
if [ ! -f "$SYSTEM_MAP_FILE" ] || [ $(cat "$SYSTEM_MAP_FILE" | wc -l) -lt 10 ]; then
	# File /boot/System.map-${KERNELRELEASE} exists, but it contains just a single line.
	# Looks like it's a Debian 11 or higher
	# Trying to use /usr/lib/debug/boot location
	SYSTEM_MAP_FILE="/usr/lib/debug/boot/System.map-${KERNELRELEASE}"

	# Use fallback location
	[ ! -f "$SYSTEM_MAP_FILE" ] && SYSTEM_MAP_FILE="/proc/kallsyms"
fi

rm -f $MAKE_LOG

if [[ $1 == "clean" ]]; then
	make -C $KERNELDIR M=$TEST_DIR clean
	rm -f $OUTPUT_FILE
	exit 0
fi

echo "CONFIGURE:"
echo "#ifndef BLKSNAP_CONFIG_H" > $OUTPUT_FILE
echo "#define BLKSNAP_CONFIG_H" >> $OUTPUT_FILE

for source_file in `ls $TEST_DIR/*.c`
do
	TEST_NAME=`basename $source_file | sed 's/\.c//g'`
	ARG_NAME=`echo $TEST_NAME | tr a-z A-Z`
	RESULT=`make -C $KERNELDIR M=$TEST_DIR $TEST_NAME.o >> \
		$MAKE_LOG 2>&1 && echo 1 || echo 0`
	echo "#define $ARG_NAME   $RESULT " >> $OUTPUT_FILE
	echo "  $ARG_NAME=$RESULT"
done

while read SYMBOL_NAME; do
	if [ -z "SYMBOL_NAME" ]; then
		continue
	fi

	echo "getting $SYMBOL_NAME address"
	MACRO_NAME="$(echo ${SYMBOL_NAME} | awk '{print toupper($0)}')_ADDR"
	SYMBOL_ADDR=$(grep " ${SYMBOL_NAME}$" "${SYSTEM_MAP_FILE}" | awk '{print $1}')
	if [ -z "$SYMBOL_ADDR" ]; then
		SYMBOL_ADDR="0"
	fi
	echo "#define $MACRO_NAME 0x$SYMBOL_ADDR" >> $OUTPUT_FILE
done < $SYMBOLS_TO_GET_ADDR_FILE

echo "#endif" >> $OUTPUT_FILE

