#ifndef _BLKSNAP_TEST_H
#define _BLKSNAP_TEST_H

#define TESTS_SUCCESS 0
#define TESTS_FAILED -EAGAIN

extern unsigned int tests_run, tests_ok;
typedef int test_ret_t;

#define mu_print(fmt, args...) do {				\
	printk("UNIT_TEST: ");					\
	printk(fmt, ##args);					\
	printk(" (%s:%s,%d)\n", __FILE__, __func__, __LINE__);	\
} while (0)

#define mu_print_fail(fmt, args...) do {			\
	printk(KERN_ALERT "UNIT_TEST: FAIL: ");			\
	printk(fmt, ##args);					\
	printk(" (%s:%s,%d)\n", __FILE__, __func__, __LINE__);	\
} while (0)

#define mu_assert(test, fmt, args...) do {	\
	if (!(test)) {				\
		mu_print_fail(fmt, ##args);	\
		return 1;			\
	}					\
} while (0)

#define mu_assert_equal(expected, actual, fmt, args...) do {	\
	if ((actual) != (expected)) {				\
		mu_print_fail(fmt, ##args);			\
		return 1;					\
	}							\
} while (0)

#define mu_assert_not_equal(expected, actual, fmt, args...) do {\
	if ((actual) == (expected)) {				\
		mu_print_fail(fmt, ##args);			\
		return 1;					\
	}							\
} while (0)

#define mu_run_test(test) do {		\
	test_ret_t ret = test();	\
	tests_run++;			\
	if (ret)			\
		return 1;		\
	else				\
		tests_ok++;		\
} while (0)

#define mu_start(test) mu_print("Start test %s", test)

#define mu_end(test) mu_print("End test %s", test)

#endif /* _BLKSNAP_TEST_H */
