#!/bin/bash

DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
. $DIR/mem-test-lib.sh

function cleanup {
  sudo rm -f $WRT_BEFORE &>/dev/null
  sudo rm -f $WRT_AFTER &>/dev/null
  sudo rm -f $COW_BEFORE &>/dev/null
  sudo rm -f $COW_AFTER &>/dev/null
  sudo rm -rf $LKM_DIR &>/dev/null || fail $? "Can't remove directory \"$LKM_DIR\""
  sudo umount $MNT &>/dev/null
  sudo umount $DEV &>/dev/null
  sudo rm -rf $MNT &>/dev/null
  sudo rmmod blk-snapshot &>/dev/null
  sudo rmmod rapidrecovery-vss &>/dev/null
}

echo Preparing machine for memory compare test...

echo Authorize yourself for sudo:
sudo /bin/true
echo

echo -n Clean up...
cleanup
echo OK

echo -n Prepare source code...
tar xf $LKM.tgz &>/dev/null || fail $? "Can't untar \"$LKM.tgz\""
echo OK

echo -n Build $LKM module...
make -C $LKM_DIR &>/dev/null || fail $? "Can't build \"$LKM\" project"
echo OK

echo We need to run following commands as root. Cache sudo...
sudo /bin/true
echo Done

echo -n Prepare test device...
sudo mkdir -p $MNT &>/dev/null || fail $? "Can't create \"$MNT\" directory"
sudo mkfs.ext4 -q $DEV &>/dev/null || fail $? "Can't make ext4 fs on \"$DEV\""
sudo mount $DEV $MNT &>/dev/null || fail $? "Can't mount \"$DEV\" to \"$MNT\""
sudo $BSCTL --create-data-store $DEV &>/dev/null
sudo $BSCTL --create-chlog-store $DEV &>/dev/null
echo OK

echo -n Load kernel module $LKM...
sudo insmod $MOD || fail $? "Can't load module \"$LKM\""
echo OK

echo -n Attach $DEV...
sudo $BSCTL -a $DEV &>/dev/null || fail $? "Can't attach \"$DEV\""
echo OK

echo -n Map store on $DEV...
sudo $BSCTL --map-data-store $DEV &>/dev/null || fail $? "Can't map data store on \"$DEV\""
sudo $BSCTL --map-metadata-store $DEV &>/dev/null || fail $? "Can't map metadata store on \"$DEV\""
sudo $BSCTL --map-chlog-store $DEV &>/dev/null || fail $? "Can't map chlog store on \"$DEV\""
echo OK

echo Preparation is done.

