#include "blksnap-test.h"
#include "blksnap-test.c"

#include "blksnap-cl-thread.h"
#include "blksnap-cl-thread.c"

#include "blksnap-cl-buffer.h"
#include "blksnap-cl-buffer.c"

#include "blksnap-cl-store.h"
#include "blksnap-cl-store.c"

#include "blksnap-volume.h"
#include "blksnap-volume.c"

#include "blksnap-bio.h"
#include "blksnap-bio.c"

#include "blksnap-bio-queue.h"
#include "blksnap-bio-queue.c"

#include "blksnap-data.h"
#include "blksnap-data.c"

#include "blksnap-bitmap.h"
#include "blksnap-bitmap.c"

#include "blksnap-block.h"
#include "blksnap-block.c"

#include "blksnap-bio-submit.h"
#include "blksnap-bio-submit.c"

#include "blksnap-cow-tree.h"
#include "blksnap-cow-tree.c"

#include "blksnap-extent.h"
#include "blksnap-extent.c"

#include "blksnap-shared.c"

#define BLOCKS_CNT	100000
#define BLOCKS_IN_PAGE	(PAGE_SIZE / sizeof(uint64_t))
#define TOTAL_PAGE_CNT	((BLOCKS_CNT / BLOCKS_IN_PAGE) \
	+ ((BLOCKS_CNT % BLOCKS_IN_PAGE) ? 1 : 0))

static unsigned long block_adds = 0;
static unsigned long write_calls = 0;
static unsigned long invalid_calls = 0;

static int writer_cb_empty(sector_t *page_buffer, unsigned int len,
			   void *data)
{
	return 0;
}

static int writer_cb_fn(sector_t *page_buffer, unsigned int len, void *data)
{
	write_calls++;
	return 0;
}

static void make_invalid(void *data)
{
	invalid_calls++;
}

static test_ret_t cl_thread_test_init_destroy(void)
{
	gfp_t gfp_mask = GFP_KERNEL;
	struct block_device *bdev;
	struct cl_thread *thread;
	struct cl_thread_opts opts = {
		.writer_cb = writer_cb_empty,
		.writer_cb_data = &write_calls,
		.invalidate_cb = make_invalid,
		.invalidate_data = &invalid_calls,
	};

	mu_start(__func__);


	bdev = kzalloc(sizeof(*bdev), gfp_mask);
	mu_assert(bdev, "Can't allocate memory");
	bdev->bd_dev = MKDEV(77, 77);

	/* Create changelog thread */
	thread = blksnap_cl_thread_init(&opts, bdev);
	mu_assert(thread, "Can't init cl thread");

	/* Destroy */
	blksnap_cl_thread_destroy(thread);

	kfree(bdev);

	mu_end(__func__);

	return 0;
}

static test_ret_t cl_thread_test_high_intencity(void)
{
	gfp_t gfp_mask = GFP_KERNEL;
	struct block_device *bdev;
	struct cl_thread *thread;
	int i;
	struct cl_thread_opts opts = {
		.writer_cb = writer_cb_fn,
		.writer_cb_data = &write_calls,
		.invalidate_cb = make_invalid,
		.invalidate_data = &invalid_calls,
	};

	mu_start(__func__);

	bdev = kzalloc(sizeof(*bdev), gfp_mask);
	mu_assert(bdev, "Can't allocate memory");
	bdev->bd_dev = MKDEV(77, 77);

	thread = blksnap_cl_thread_init(&opts, bdev);
	mu_assert(thread, "Can't init cl thread");

	for (i = 0; i < BLOCKS_CNT; i++) {
		if (!blksnap_cl_thread_add_block(thread, 1))
			block_adds++;
	}

	blksnap_cl_thread_destroy(thread);

	// mu_print("block_adds=%lu blocks_cnt=%u", block_adds, BLOCKS_CNT);
	mu_assert_equal(block_adds, BLOCKS_CNT,
			"Wrong value of added blocks [%lu != %u]",
			block_adds, BLOCKS_CNT);
	// mu_print("write_calls=%lu total_page_cnt=%lu", write_calls, TOTAL_PAGE_CNT);
	mu_assert_equal(write_calls, TOTAL_PAGE_CNT,
			"Wrong amount of write calls [%lu, %lu]",
			write_calls, TOTAL_PAGE_CNT);
	// mu_print("invalid_calls=%lu", invalid_calls);
	mu_assert_equal(invalid_calls, 0,
			"There are errors during adding blocks [%lu]",
			invalid_calls);

	kfree(bdev);

	mu_end(__func__);

	return 0;
}

static test_ret_t all_tests()
{
	int ret;

	/* init common resources */
	ret = blksnap_cl_cmd_cache_create();
	mu_assert(!ret, "Create cmd_cache");

	mu_run_test(cl_thread_test_init_destroy);
	mu_run_test(cl_thread_test_high_intencity);

	/* deinit common resources */
	blksnap_cl_cmd_cache_destroy();

	return 0;
}
