#include "blksnap-test.h"
#include "blksnap-test.c"

#include "blksnap-shared.h"
#include "blksnap-shared.c"

#include "blksnap-hash.c"

static test_ret_t hash_sum_test_same_hashes(void)
{
	__u32 first_hash[BLKSNAP_HASH_DIGEST_WORDS];
	__u32 second_hash[BLKSNAP_HASH_DIGEST_WORDS];
	int ret;
	const u8 * data = "123456789";

	mu_start(__func__);

	ret = calculate_hash(data, 9, (__u32 *) &first_hash);
	mu_assert(!ret, "ERROR: Unable to calculate hash sum");
	ret = calculate_hash(data, 9, (__u32 *) &second_hash);
	mu_assert(!ret, "ERROR: Unable to calculate hash sum");

	mu_assert_equal(first_hash[0], second_hash[0],
			"ERROR: hash sums differs in 0 section [%x != %x]",
			first_hash[0], second_hash[0]);
	mu_assert_equal(first_hash[1], second_hash[1],
			"ERROR: hash sums differs in 1st section [%x != %x]",
			first_hash[1], second_hash[1]);
	mu_assert_equal(first_hash[2], second_hash[2],
			"ERROR: hash sums differs in 2nd section [%x != %x]",
			first_hash[2], second_hash[2]);
	mu_assert_equal(first_hash[3], second_hash[3],
			"ERROR: hash sums differs in 3rd section [%x != %x]",
			first_hash[3], second_hash[3]);
	mu_end(__func__);

	return 0;
}

static test_ret_t hash_sum_test_diff_hashes(void)
{
	__u32 first_hash[BLKSNAP_HASH_DIGEST_WORDS];
	__u32 second_hash[BLKSNAP_HASH_DIGEST_WORDS];
	const u8 * data1 = "123456789";
	const u8 * data2 = "987654321";
	int ret;

	mu_start(__func__);

	ret = calculate_hash(data1, 9, (__u32 *) &first_hash);
	mu_assert(!ret, "ERROR: Unable to calculate hash sum");
	ret = calculate_hash(data2, 9, (__u32 *) &second_hash);
	mu_assert(!ret, "ERROR: Unable to calculate hash sum");

	mu_assert((first_hash[0] != second_hash[0]) ||
		  (first_hash[1] != second_hash[1]) ||
		  (first_hash[2] != second_hash[2]) ||
		  (first_hash[3] != second_hash[3]),
		  "ERROR: hash sums are the same [%x%x%x%x]",
		  first_hash[0], first_hash[1], first_hash[2], first_hash[3]);
	mu_end(__func__);

	return 0;
}

static test_ret_t hash_sum_test_empty_hashes(void)
{
	__u32 first_hash[BLKSNAP_HASH_DIGEST_WORDS];
	__u32 second_hash[BLKSNAP_HASH_DIGEST_WORDS];
	const u8 * data = "";
	int ret;

	mu_start(__func__);

	ret = calculate_hash(data, 0, (__u32 *) &first_hash);
	mu_assert(!ret, "ERROR: Unable to calculate hash sum");
	ret = calculate_hash(data, 0, (__u32 *) &second_hash);
	mu_assert(!ret, "ERROR: Unable to calculate hash sum");

	mu_assert_equal(first_hash[0], second_hash[0],
			"ERROR: hash sums differs in 0 section [%x != %x]",
			first_hash[0], second_hash[0]);
	mu_assert_equal(first_hash[1], second_hash[1],
			"ERROR: hash sums differs in 1 section [%x != %x]",
			first_hash[1], second_hash[1]);
	mu_assert_equal(first_hash[2], second_hash[2],
			"ERROR: hash sums differs in 2 section [%x != %x]",
			first_hash[2], second_hash[2]);
	mu_assert_equal(first_hash[3], second_hash[3],
			"ERROR: hash sums differs in 0 section [%x != %x]",
			first_hash[3], second_hash[3]);

	mu_end(__func__);

	return 0;
}

static test_ret_t all_tests()
{
	mu_run_test(hash_sum_test_same_hashes);
	mu_run_test(hash_sum_test_diff_hashes);
	mu_run_test(hash_sum_test_empty_hashes);

	return 0;
}
