#include <stdio.h>
#include <stdarg.h>
#include <errno.h>
#include <inttypes.h>
#include <linux/types.h>

#include "blksnap.h"
#include "bsctl-error.h"

int bsctl_error_set(struct bsctl_error *error, int err, const char *fmt, ...)
{
	va_list ap;

	if (!error)
		goto ret;

	error->err = err;

	va_start(ap, fmt);
	vsnprintf(error->msg, BSCTL_ERROR_MAX_LEN, fmt, ap);
	va_end(ap);

ret:
	return err;
}

int bsctl_error_ioctl(struct bsctl_error *error, int err,
		      const char *dev_path)
{
	switch (err) {
	case ENOMEM:
		bsctl_err_set(error, err, "failed to allocate memory");
		break;
	case ENODEV:
		bsctl_err_set(error, err,
			      "device %s not found. Possible it is not attached",
			      dev_path);
		break;
	case EBUSY:
		if (dev_path != NULL)
			bsctl_err_set(error, err,
				      "device %s is busy, please try again later",
				      dev_path);
		else
			bsctl_err_set(error, err,
				      "kernel module is busy, please try again later");
		break;
	case EEXIST:
		bsctl_err_set(error, err, "device %s already exists",
			      dev_path);
		break;
	case ENXIO:
		bsctl_err_set(error, err,
			      "device %s is not suitable for snapshot",
			      dev_path);
		break;
	case EPERM:
		bsctl_err_set(error, err,
			      "permission denied, you must be root");
		break;
	case E2BIG:
		bsctl_err_set(error, err,
			      "multiply request is not supported");
		break;
	case EVERSION:
		bsctl_err_set(error, err,
			      "version mismatch with kernel module");
		break;
	case EHASSNAP:
		bsctl_err_set(error, err,
			      "%s: a snapshot is in progress", dev_path);
		break;
	case ENOSNAP:
		bsctl_err_set(error, err, "%s: no snapshot found",
			      dev_path);
		break;
	case EDIRTY:
		bsctl_err_set(error, err,
			      "operation denied, device %s is dirty",
			      dev_path);
		break;
	case EHASDATA:
		bsctl_err_set(error, err,
			      "device %s has already mapped data store",
			      dev_path);
		break;
	case ECLSNOMAP:
		bsctl_err_set(error, err,
			      "device %s has no mapped changelog store",
			      dev_path);
		break;
	case EDTSNOMAP:
		bsctl_err_set(error, err,
			      "device %s has no mapped data store",
			      dev_path);
		break;
	case ECLSFULL:
		bsctl_err_set(error, err,
			      "full changelog store is detected on device %s",
			      dev_path);
		break;
	case EMDSNOMAP:
		bsctl_err_set(error, err,
			      "device %s has no mapped metadata store",
			      dev_path);
		break;
	case ESIZESTORE:
		bsctl_err_set(error, err,
			      "datastore for device %s has incorrect size", dev_path);
		break;
	case ESIZEVOLUME:
		bsctl_err_set(error, err,
			      "stored volume size for device %s and actual size mismatch", dev_path);
		break;
	default:
		bsctl_err_set(error, err,
			      "operation failed (error code: %d)", err);
	}

	return BLKSNAP_ERROR_TO_USER(err);
}
