#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <linux/types.h>
#include "blksnap.h"
#include "bsctl-error.h"
#include "bsctl-ioctl.h"

void bsctl_init_ioctl(struct blksnap_ioctl *bsi)
{
	memset(bsi, 0, sizeof(struct blksnap_ioctl));
	bsi->version = BLKSNAP_IOCTL_VERSION;
	bsi->size = sizeof(struct blksnap_ioctl);
}

int bsctl_send_ioctl(unsigned int cmd, struct blksnap_ioctl *bsi,
		     const char *dev_path, struct bsctl_error *err)
{
	struct stat statbuf;
	int fd, ret = 0;
	static char control_path[BLKSNAP_NAME_SIZE];

	snprintf(control_path, sizeof(control_path), "/dev/%s", BLKSNAP_NAME);

	if (stat(control_path, &statbuf) == -1)
		return bsctl_error_set(err, errno,
				       "failed to open %s. Possible the module is not loaded: %s",
				       control_path, strerror(errno));

	if ((fd = open(control_path, O_RDONLY | O_NONBLOCK)) < 0)
		return bsctl_error_set(err, errno, "failed to open %s: %s",
				       control_path, strerror(errno));

	if (ioctl(fd, cmd, bsi) == -1)
		ret = bsctl_error_ioctl(err, errno, dev_path);

	close(fd);
	return BLKSNAP_ERROR_TO_USER(ret);
}
