#ifndef _BSCTL_IOCTL_H
#define _BSCTL_IOCTL_H

#include "blksnap.h"
#include "bsctl-error.h"

void bsctl_init_ioctl(struct blksnap_ioctl *bsi);

int bsctl_send_ioctl(unsigned int cmd, struct blksnap_ioctl *bsi,
		     const char *dev_path, struct bsctl_error *err);

/* Requirements for buffer size of dynamic-sized ioctls
 * 1) ioctls must have common header
 * 2) ioctl buffer must be able to contain all dynamic elements
 * 3) buffer size can't be less than sizeof(struct blksnap_ioctl) because buffer
 *    has type <struct blksnap_ioctl *> and try to free() buffer with size less
 *    than sizeof(struct blksnap_ioctl) will interrupt process with SIGSEG.
 *    From the other side, module copies sizeof(struct blksnap_ioctl) bytes on
 *    ioctl receiving (however, it's not obligatory behavior).
 * NOTE: It is possible to avoid using <struct blksnap_ioctl *> pointer. We
 *       could use common header as a base part of each ioctl. But for now pros
 *       of such approach are minor.
 */
#define BSCTL_CALC_IOCTL_SIZE(IOCTL_TYPE, oversize)	\
	(MAX(sizeof(struct blksnap_ioctl),		\
	     BLKSNAP_IOCTL_HEADER_SIZE + sizeof(IOCTL_TYPE) + (oversize)))

#endif /* _BSCTL_IOCTL_H */
