#include <stdio.h>
#include <stdarg.h>
#include <stdbool.h>
#include <syslog.h>
#include <inttypes.h>

#include "bsctl-log.h"
#include "bsctl-error.h"

#define LIB_SYSLOG_PRI	LOG_INFO

static int log_default = LIB_SYSLOG_PRI;
static int *log_priority_ptr = &log_default;
static bool syslog_opened = false;
static bool skip_stdout = true;

void bsctl_set_log_priority(int *log_var)
{
	int syslog_pri = *log_var;

	log_priority_ptr = log_var;

	/* raise log level for syslog */
	if ((syslog_pri & LOG_PRIMASK) < LOG_DEBUG)
		syslog_pri++;

	setlogmask(LOG_UPTO(syslog_pri));
	skip_stdout = false;
}

void bsctl_log(int log_pri, FILE *out, const char *fmt, ...)
{
	va_list ap;
	char buf[BSCTL_ERROR_MAX_LEN];


	if (!syslog_opened) {
		openlog(SYSLOG_PREFIX, SYSLOG_OPTIONS, SYSLOG_FACILITY);
		syslog_opened = true;
		if (skip_stdout)
			setlogmask(LOG_UPTO((*log_priority_ptr & LOG_PRIMASK)));
	}

	va_start(ap, fmt);
	vsnprintf(buf, BSCTL_ERROR_MAX_LEN, fmt, ap);
	va_end(ap);

	syslog(log_pri, "%s", buf);

	if (skip_stdout || log_pri > (*log_priority_ptr & LOG_PRIMASK))
		return;

	fprintf(out, "%s\n", buf);
}

