#ifndef BSCTL_LOG_H
#define BSCTL_LOG_H

#include <syslog.h>

/* Default options for syslog
 * SYSLOG_PREFIX   - a prefix for each message sent to syslog
 * SYSLOG_OPTIONS  - options passed when syslog connection is created
 *		     For list of options please see "man 3 syslog"
 * SYSLOG_FACILITY - facility that must be used in syslog.conf to
 *		     catch messages from bsctl
 *		     For list of facilities please see "man 3 syslog"
 */
#define SYSLOG_PREFIX   "bsctl"
#define SYSLOG_OPTIONS  LOG_PID
#define SYSLOG_FACILITY LOG_LOCAL6

/* set logging priority, please see sys/syslog.h for reference */
void bsctl_set_log_priority(int *log_var);

/* pass a message to syslog and print it to if priority is
 * NOTE: this function adds an '\n' at the end of fmt string */
void bsctl_log(int priority, FILE *out, const char *fmt, ...);

#define bsctl_err(priority, fmt, ...) \
	bsctl_log((priority), stderr, fmt, ##__VA_ARGS__)

#define bsctl_msg(priority, fmt, ...) \
	bsctl_log((priority), stdout, fmt, ##__VA_ARGS__)

#endif

