#ifndef BSCTL_SHARE_H
#define BSCTL_SHARE_H

#include <stdbool.h>

#define SECTOR_SHIFT		9
#define PAGE_SHIFT		12
#define PAGE_SECTORS_SHIFT	(PAGE_SHIFT - SECTOR_SHIFT)
#define PAGE_SECTORS		(1 << PAGE_SECTORS_SHIFT)
#define SECTOR_SIZE		(1 << SECTOR_SHIFT)
#define PAGE_SIZE		(1 << PAGE_SHIFT)

/* Default significant values. All values are selected empirically */

/* How many pages is allocated to store CoW data in RAM */
#define STORE_DATA_NPAGES	2048		/* # of 4k pages (~8MB) */
/* Size of data store file to keep CoW data on disk
 * data_store_file_size(disk) = total_size(disk) * STORE_DATA_RATE
 * Use to create data store file w/o --store-size parameter
 */
#define STORE_DATA_RATE		0.15		/* part of total volume size */
/* Size of changelog store file to keep changelog data thru reboot
 * or detach/attach
 * chlog_store_file_size(disk) = total_size(disk) * STORE_CHLOG_SIZE
 * Use to create chlog store file w/o --store-size parameter
 */
#define STORE_CHLOG_RATE	(1.0 / 512.0)	/* part of total volume size */

#define UNUSED(x) x __attribute__((unused))

#ifndef sector_t
#define sector_t uint64_t
#endif

/* struct to collect statistic from changelog ioctl */
struct changelog_stat {
	unsigned int total_chunks;
	unsigned int total_extents;
	unsigned int total_blocks;
};

/* calculate how much sub-elements of object should be allocated
 * to fit the whole object to a target size
 * ATTENTION: sub-element is a pointer in a structure of an object
 */
static inline size_t elems_in_size(size_t max_size, size_t object_size,
				   size_t element_size)
{
	if (max_size <= object_size)
		return 0;
	return (max_size - object_size) / element_size;
}

#endif

