#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <errno.h>
#include <string.h>
#include <libgen.h>

#include <sys/stat.h>

#include "version.h"

#define PAGE_SIZE 4096

void usage(int ret, char *errormsg)
{
	if (ret != 0 && errormsg != NULL)
		fprintf(stderr, "Error: %s\n", errormsg);

	printf("Usage: randomwrite <filename> [<iterations>]\n"
	       "  where:\n"
	       "         filename   - name of file to be written randomly\n"
	       "         iterations - number of write iterations,"
	       " -1 - infinite loop\n"
	       "WARNING! Data of given file will be completely broken with\n"
	       "         this command! Be careful!\n");
	exit(ret);
}

static void version(char *name)
{
	fprintf(stdout, "%s version %s\n", basename(name), BLKSNAP_VERSION);
}

int main(int argc, char **argv)
{
	unsigned long blocks = 0;
	long iter = -1;
	struct stat st;
	FILE *fp;
	bool randomly = true;
	char buf[21];

	version(argv[0]);

	if (argc < 2) {
		usage(0, NULL);
	}
	if (stat(argv[1], &st))
		usage(errno, strerror(errno));
	if (!st.st_size)
		usage(EINVAL, "File is too short");
	if (st.st_size <= PAGE_SIZE)
		randomly = 0;
	else
		blocks = (st.st_size - 1) / PAGE_SIZE + 1;


	fp = fopen(argv[1], "r+");
	if (fp == NULL)
		usage(errno, strerror(errno));


	if (argc > 2) {
		errno = 0; /* reset errno as strtol() MAY set this varibale */
		iter = strtol(argv[2], NULL, 10);
		if (!iter && errno) {
			int ret = errno;
			fclose(fp);
			usage(ret, strerror(ret));
		}
	}

	while (iter != 0) {
		unsigned long block = 0;
		size_t sz;
		if (randomly)
			block = random() % blocks;

		sprintf(buf, "%lu", block);
		fseek(fp, block * PAGE_SIZE, SEEK_SET);
		sz = fwrite(buf, strlen(buf), 1, fp);
		if (sz != 1)
			perror("couldn't write file (%lu)");

		if (iter > 0)
			iter--;
	}

	fclose(fp);

	return(0);
}
